# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_slice_widget.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SliceWidget(object):
    def setupUi(self, SliceWidget):
        SliceWidget.setObjectName("SliceWidget")
        SliceWidget.resize(554, 642)
        self.gridLayout = QtWidgets.QGridLayout(SliceWidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.graphicsView = GraphicsLayoutWidget(SliceWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.graphicsView.sizePolicy().hasHeightForWidth())
        self.graphicsView.setSizePolicy(sizePolicy)
        self.graphicsView.setObjectName("graphicsView")
        self.verticalLayout.addWidget(self.graphicsView)
        self.controlsWidget = QtWidgets.QWidget(SliceWidget)
        self.controlsWidget.setMaximumSize(QtCore.QSize(16777215, 25))
        self.controlsWidget.setObjectName("controlsWidget")
        self.layout_index_slider = QtWidgets.QHBoxLayout(self.controlsWidget)
        self.layout_index_slider.setContentsMargins(20, 2, 15, 2)
        self.layout_index_slider.setObjectName("layout_index_slider")
        self.labelImageSeriesNumber = QtWidgets.QLabel(self.controlsWidget)
        self.labelImageSeriesNumber.setText("")
        self.labelImageSeriesNumber.setObjectName("labelImageSeriesNumber")
        self.layout_index_slider.addWidget(self.labelImageSeriesNumber)
        self.seriesSlider = QtWidgets.QSlider(self.controlsWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.seriesSlider.sizePolicy().hasHeightForWidth())
        self.seriesSlider.setSizePolicy(sizePolicy)
        self.seriesSlider.setMinimumSize(QtCore.QSize(200, 0))
        self.seriesSlider.setOrientation(QtCore.Qt.Horizontal)
        self.seriesSlider.setInvertedAppearance(False)
        self.seriesSlider.setInvertedControls(False)
        self.seriesSlider.setTickPosition(QtWidgets.QSlider.TicksAbove)
        self.seriesSlider.setTickInterval(1)
        self.seriesSlider.setObjectName("seriesSlider")
        self.layout_index_slider.addWidget(self.seriesSlider)
        self.labelSliceNumber = QtWidgets.QLabel(self.controlsWidget)
        self.labelSliceNumber.setText("")
        self.labelSliceNumber.setObjectName("labelSliceNumber")
        self.layout_index_slider.addWidget(self.labelSliceNumber)
        self.sliderSlice = QtWidgets.QSlider(self.controlsWidget)
        self.sliderSlice.setMaximumSize(QtCore.QSize(16777215, 10))
        self.sliderSlice.setOrientation(QtCore.Qt.Horizontal)
        self.sliderSlice.setTickInterval(0)
        self.sliderSlice.setObjectName("sliderSlice")
        self.layout_index_slider.addWidget(self.sliderSlice)
        self.pushButtonScrollLeft = QtWidgets.QPushButton(self.controlsWidget)
        self.pushButtonScrollLeft.setStyleSheet("border:none")
        self.pushButtonScrollLeft.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/scroll_arrow_left.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButtonScrollLeft.setIcon(icon)
        self.pushButtonScrollLeft.setIconSize(QtCore.QSize(15, 15))
        self.pushButtonScrollLeft.setObjectName("pushButtonScrollLeft")
        self.layout_index_slider.addWidget(self.pushButtonScrollLeft)
        self.pushButtonScrollRight = QtWidgets.QPushButton(self.controlsWidget)
        self.pushButtonScrollRight.setStyleSheet("border:none")
        self.pushButtonScrollRight.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/scroll_arrow_right.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon1.addPixmap(QtGui.QPixmap(":/icons/scroll_arrow_right_active.png"), QtGui.QIcon.Active, QtGui.QIcon.On)
        self.pushButtonScrollRight.setIcon(icon1)
        self.pushButtonScrollRight.setIconSize(QtCore.QSize(15, 15))
        self.pushButtonScrollRight.setObjectName("pushButtonScrollRight")
        self.layout_index_slider.addWidget(self.pushButtonScrollRight)
        self.pushButtonManageVolumes = QtWidgets.QPushButton(self.controlsWidget)
        self.pushButtonManageVolumes.setStyleSheet("border:none")
        self.pushButtonManageVolumes.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/settings.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButtonManageVolumes.setIcon(icon2)
        self.pushButtonManageVolumes.setIconSize(QtCore.QSize(15, 15))
        self.pushButtonManageVolumes.setFlat(False)
        self.pushButtonManageVolumes.setObjectName("pushButtonManageVolumes")
        self.layout_index_slider.addWidget(self.pushButtonManageVolumes)
        self.verticalLayout.addWidget(self.controlsWidget)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 0, 1, 1)

        self.retranslateUi(SliceWidget)
        QtCore.QMetaObject.connectSlotsByName(SliceWidget)

    def retranslateUi(self, SliceWidget):
        _translate = QtCore.QCoreApplication.translate
        SliceWidget.setWindowTitle(_translate("SliceWidget", "Form"))
        self.labelSliceNumber.setAccessibleName(_translate("SliceWidget", "sliceNumber"))
from pyqtgraph import GraphicsLayoutWidget
from . import resources_rc



# Copyright 2016 Medical Research Council Harwell.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# @author James Brown <james.brown@har.mrc.ac.uk>

import subprocess as sp
import numpy as np
import re
from tempfile import NamedTemporaryFile

minc_dtypes = {'unsigned': {'byte': np.uint8, 'short': np.uint16, 'float': np.float32},
               'signed': {'byte': np.int8, 'short': np.int16, 'float': np.float32}}

def minc_to_numpy(minc_file):

    info = minc_info(minc_file)
    if not info:
        return False
    mrsg = MincRawSliceGenerator(minc_file, info)
    vol = mrsg.volume
    return vol


def mincstats_to_numpy(minc_file):

    info = minc_info(minc_file)
    if not info:
        return False
    info['dtype'] = 'float'  # have to force it, because mincinfo is hopeless
    info['np_dtype'] = np.float32
    mrsg = MincRawSliceGenerator(minc_file, info)
    vol = mrsg.volume
    return vol


class SliceGenerator(object):

    def __init__(self, recon):
        self.recon = recon
        self.slice_index = 0

    def slices(self):
        """The slices method should yield xy image slices from a memory mapped numpy array."""
        raise NotImplementedError("Ths method needs overriding")

    def dtype(self):
        """The dtype method should return the datatype of the memory mapped numpy array"""
        raise NotImplementedError("Ths method needs overriding")

    def shape(self):
        """The shape method should return the shape of the memory mapped numpy array in x, y, z order."""
        raise NotImplementedError("Ths method needs overriding")

class MincRawSliceGenerator(SliceGenerator):
    """The MincRawSliceGenerator class extends SliceGenerator, yielding slices from a single MINC (Medical Image NetCDF)
    file, having been dumped to a temporary raw file via minctoraw. mincinfo is used to determine the file type/dimensions
    """

    def __init__(self, recon, info):
        """The constructor takes a recon path as an argument, and dumps the MINC file to a temporary raw file. The raw file
        is then memory mapped using numpy, from which slices are yielded.

        :param recon: a path to a MINC file.
        """
        super(MincRawSliceGenerator, self).__init__(recon)
        self.ext = 'mnc'

        tmp_file = NamedTemporaryFile() # TemporaryFile() seems not to work with Python3.4

        sp.call(['mincextract', '-{}'.format(info['dtype']),
                             '-{}'.format(info['sign']), recon], stdout=tmp_file)

        self.volume = np.fromfile(tmp_file.name, dtype=info['np_dtype']).reshape(info['shape'])

    def slices(self, start=0):
        """Slices are yielded one slice at a time from the memory mapped numpy array
        """
        try:
            for i in range(self.volume.shape[0]):
                yield self.volume[i, :, :]
        except Exception:
            raise CorruptReconError("Error yielding slices from MINC file")

    def dtype(self):
        """Overrides the superclass to return the data type of the MINC file i.e. 8 bit/16 bit.
        """
        return self.volume.dtype

    def shape(self):
        """Overrides the superclass to return the shape of the MINC file.
        """
        return self.volume.shape[::-1]


def minc_info(recon):

    try:
        info = sp.check_output(['mincinfo', recon], universal_newlines=True)
    except OSError as e:
        raise OSError("Minc tools not installed\n{}".format(e))
    #info = str(info)

    info_dict = {}
    dims = []
    for line in info.splitlines():

        if 'image:' in line:  # strip non alphanumeric characters

            # Get range
            min_max = re.findall('\d+ to \d+', line)[0]
            info_dict['min'], info_dict['max'] = int(min_max.split()[0]), int(min_max.split()[2])

            regex = re.compile('[^a-zA-Z]')
            info_dict['sign'] = regex.sub('', line.split()[1])
            info_dict['dtype'] = regex.sub('', line.split()[2])

            try:
                info_dict['np_dtype'] = minc_dtypes[info_dict['sign']][info_dict['dtype']]
            except KeyError:
                return None

        elif 'dimensions' not in line and any(space in line for space in ['xspace', 'yspace', 'zspace']):
            spacing = line.split()
            dims.append(int(spacing[1]))
            info_dict['voxel_size'] = float(spacing[2]) * 1000  # in microns

    info_dict['shape'] = tuple(dims)  # zspace, yspace, xspace
    return info_dict


class ReconFormatError(Exception):

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)


class CorruptReconError(Exception):

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)



from nose.tools import assert_equals, nottest
import sys
from os.path import join, dirname
sys.path.insert(0, join(dirname(__file__), '..'))
from annotations.impc_xml import load_xml

import os

current_dir = os.path.dirname(os.path.realpath(__file__))

test_xml_path = join(current_dir, 'Inpp5e-MBM-popavg.xml')

load_xml(test_xml_path)



from vpv.ui.views.ui_consoletab import Ui_console
from PyQt5 import QtCore, QtGui
# from pyqtgraph import console
from vpv.common import Layers
import skimage
import scipy
import SimpleITK as sitk



import os
os.environ['QT_API'] = 'pyqt'
import sip
sip.setapi("QString", 2)
sip.setapi("QVariant", 2)
from PyQt5.QtGui  import *
from PyQt5.QtWidgets import QWidget
# Import the console machinery from ipython

from qtconsole.rich_jupyter_widget import RichJupyterWidget
from qtconsole.inprocess import QtInProcessKernelManager
from IPython.lib import guisupport

class Console(QWidget):
    console_command_executed = QtCore.pyqtSignal()

    def __init__(self, mainwindow, vpv):
        super(Console, self).__init__(mainwindow)
        self.ui = Ui_console()
        self.ui.setupUi(self)
        self.vpv = vpv
        variables = {
             "vpv_viewer": self.vpv,
             "current_volume": self.current_volume,
             "volume_ids": self.volume_ids,
             "get_volume": self.get_volume,
             "print_process_id": print_process_id
        }

        widget = ExampleWidget(variables, parent=vpv)
        self.ui.mainLayout.addWidget(widget)

    def activate_tab(self):
        pass

    def current_volume(self):
        return self.vpv.current_view.layers[Layers.vol1]

    def volume_ids(self):
        return self.vpv.model.volume_id_list()

    def get_volume(self, id_):
        return self.vpv.model.getvol(id_)._arr_data

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Enter:
            print('event')
            self.vpv.on_console_enter_pressesd()


class QIPythonWidget(RichJupyterWidget):
    """ Convenience class for a live IPython console widget.
    We can replace the standard banner using the customBanner argument"""

    def __init__(self, customBanner='VPV console', parent=None, *args,**kwargs):
        if customBanner!=None: self.banner=customBanner
        super(QIPythonWidget, self).__init__(*args,**kwargs)
        self.kernel_manager = kernel_manager = QtInProcessKernelManager()
        kernel_manager.start_kernel()
        kernel_manager.kernel.gui = 'qt4'
        self.kernel_client = kernel_client = self._kernel_manager.client()
        kernel_client.namespace = parent
        kernel_client.start_channels()

        def stop():
            kernel_client.stop_channels()
            kernel_manager.shutdown_kernel()
            guisupport.get_app_qt4().exit()
        self.exit_requested.connect(stop)

    def pushVariables(self, variableDict):
        """ Given a dictionary containing name / value pairs, push those variables to the IPython console widget """
        self.kernel_manager.kernel.shell.push(variableDict)

    def clearTerminal(self):
        """ Clears the terminal """
        self._control.clear()

    def printText(self, text):
        """ Prints some plain text to the console """
        self._append_plain_text(text)

    # def executeCommand(self, command):
    #     """ Execute a command in the frame of the console widget """
    #
    #     a = self._execute(command, False)
    #     print(a)


class ExampleWidget(QWidget):
    """ Main GUI Widget including a button and IPython Console widget inside vertical layout """
    def __init__(self, variables, parent=None):
        super(ExampleWidget, self).__init__(None)
        layout = QVBoxLayout(self)
        self.button = QPushButton('Another widget')
        ipyConsole = QIPythonWidget(parent=parent)
        #layout.addWidget(self.button)
        layout.addWidget(ipyConsole)
        # This allows the variable foo and method print_process_id to be accessed from the ipython console
        ipyConsole.pushVariables(variables)
        ipyConsole.printText("""Some functions to use. See docs for a full list:
        current_volume(): gets the the currently selected volume as a numpy array
        volume_ids(): gets a list of all available volume ids
        get_volume(volume_id): gets a volume as a numpy array
        """)

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Return:
            print('event')
            self.parent.on_console_enter_pressesd

def print_process_id():
    print('Process ID is:', os.getpid())





# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_datatab.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_data(object):
    def setupUi(self, data):
        data.setObjectName("data")
        data.resize(761, 857)
        self.verticalLayout = QtWidgets.QVBoxLayout(data)
        self.verticalLayout.setObjectName("verticalLayout")
        self.layoutVolume = QtWidgets.QVBoxLayout()
        self.layoutVolume.setObjectName("layoutVolume")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.label_13 = QtWidgets.QLabel(data)
        self.label_13.setObjectName("label_13")
        self.verticalLayout_6.addWidget(self.label_13)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.checkBoxLeftView = QtWidgets.QCheckBox(data)
        self.checkBoxLeftView.setText("")
        self.checkBoxLeftView.setChecked(True)
        self.checkBoxLeftView.setObjectName("checkBoxLeftView")
        self.horizontalLayout_8.addWidget(self.checkBoxLeftView)
        self.checkBoxCentralView = QtWidgets.QCheckBox(data)
        self.checkBoxCentralView.setText("")
        self.checkBoxCentralView.setChecked(True)
        self.checkBoxCentralView.setObjectName("checkBoxCentralView")
        self.horizontalLayout_8.addWidget(self.checkBoxCentralView)
        self.checkBoxRightView = QtWidgets.QCheckBox(data)
        self.checkBoxRightView.setText("")
        self.checkBoxRightView.setChecked(True)
        self.checkBoxRightView.setObjectName("checkBoxRightView")
        self.horizontalLayout_8.addWidget(self.checkBoxRightView)
        self.verticalLayout_6.addLayout(self.horizontalLayout_8)
        self.line_6 = QtWidgets.QFrame(data)
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_6.setObjectName("line_6")
        self.verticalLayout_6.addWidget(self.line_6)
        self.label = QtWidgets.QLabel(data)
        self.label.setObjectName("label")
        self.verticalLayout_6.addWidget(self.label)
        self.horizontalLayoutExtraButtons = QtWidgets.QHBoxLayout()
        self.horizontalLayoutExtraButtons.setObjectName("horizontalLayoutExtraButtons")
        self.pushButtonManagerRed = QtWidgets.QPushButton(data)
        self.pushButtonManagerRed.setText("")
        self.pushButtonManagerRed.setObjectName("pushButtonManagerRed")
        self.horizontalLayoutExtraButtons.addWidget(self.pushButtonManagerRed)
        self.pushButtonManagerBlue = QtWidgets.QPushButton(data)
        self.pushButtonManagerBlue.setText("")
        self.pushButtonManagerBlue.setObjectName("pushButtonManagerBlue")
        self.horizontalLayoutExtraButtons.addWidget(self.pushButtonManagerBlue)
        self.pushButtonManagerGreen = QtWidgets.QPushButton(data)
        self.pushButtonManagerGreen.setText("")
        self.pushButtonManagerGreen.setObjectName("pushButtonManagerGreen")
        self.horizontalLayoutExtraButtons.addWidget(self.pushButtonManagerGreen)
        self.verticalLayout_6.addLayout(self.horizontalLayoutExtraButtons)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.pushButtonManagerOrange = QtWidgets.QPushButton(data)
        self.pushButtonManagerOrange.setText("")
        self.pushButtonManagerOrange.setObjectName("pushButtonManagerOrange")
        self.horizontalLayout_9.addWidget(self.pushButtonManagerOrange)
        self.pushButtonManagerGrey = QtWidgets.QPushButton(data)
        self.pushButtonManagerGrey.setText("")
        self.pushButtonManagerGrey.setObjectName("pushButtonManagerGrey")
        self.horizontalLayout_9.addWidget(self.pushButtonManagerGrey)
        self.pushButtonManagerCyan = QtWidgets.QPushButton(data)
        self.pushButtonManagerCyan.setText("")
        self.pushButtonManagerCyan.setObjectName("pushButtonManagerCyan")
        self.horizontalLayout_9.addWidget(self.pushButtonManagerCyan)
        self.verticalLayout_6.addLayout(self.horizontalLayout_9)
        self.horizontalLayout.addLayout(self.verticalLayout_6)
        self.line_7 = QtWidgets.QFrame(data)
        self.line_7.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_7.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_7.setObjectName("line_7")
        self.horizontalLayout.addWidget(self.line_7)
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.checkBoxLinkViews = QtWidgets.QCheckBox(data)
        self.checkBoxLinkViews.setChecked(True)
        self.checkBoxLinkViews.setObjectName("checkBoxLinkViews")
        self.verticalLayout_7.addWidget(self.checkBoxLinkViews)
        self.checkBox6Views = QtWidgets.QCheckBox(data)
        self.checkBox6Views.setObjectName("checkBox6Views")
        self.verticalLayout_7.addWidget(self.checkBox6Views)
        self.horizontalLayout.addLayout(self.verticalLayout_7)
        self.line_8 = QtWidgets.QFrame(data)
        self.line_8.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_8.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_8.setObjectName("line_8")
        self.horizontalLayout.addWidget(self.line_8)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.label_2 = QtWidgets.QLabel(data)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_9.addWidget(self.label_2)
        self.comboBoxOrientation = QComboBoxIgnoreSCroll(data)
        self.comboBoxOrientation.setObjectName("comboBoxOrientation")
        self.comboBoxOrientation.addItem("")
        self.comboBoxOrientation.addItem("")
        self.comboBoxOrientation.addItem("")
        self.comboBoxOrientation.addItem("")
        self.verticalLayout_9.addWidget(self.comboBoxOrientation)
        self.verticalLayout_5.addLayout(self.verticalLayout_9)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.labelNewVersion = QtWidgets.QLabel(data)
        self.labelNewVersion.setText("")
        self.labelNewVersion.setObjectName("labelNewVersion")
        self.horizontalLayout_4.addWidget(self.labelNewVersion)
        self.verticalLayout_5.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.pushButtonScaleBarColor = QtWidgets.QPushButton(data)
        self.pushButtonScaleBarColor.setObjectName("pushButtonScaleBarColor")
        self.horizontalLayout_5.addWidget(self.pushButtonScaleBarColor)
        self.pushButtonScreenShot = QtWidgets.QPushButton(data)
        self.pushButtonScreenShot.setStyleSheet("border: none")
        self.pushButtonScreenShot.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/Camera_font_awesome.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButtonScreenShot.setIcon(icon)
        self.pushButtonScreenShot.setIconSize(QtCore.QSize(24, 24))
        self.pushButtonScreenShot.setObjectName("pushButtonScreenShot")
        self.horizontalLayout_5.addWidget(self.pushButtonScreenShot)
        self.verticalLayout_5.addLayout(self.horizontalLayout_5)
        self.horizontalLayout.addLayout(self.verticalLayout_5)
        self.layoutVolume.addLayout(self.horizontalLayout)
        self.line_5 = QtWidgets.QFrame(data)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_5.setObjectName("line_5")
        self.layoutVolume.addWidget(self.line_5)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.labelVolume = QtWidgets.QLabel(data)
        self.labelVolume.setStyleSheet("background-color:rgb(102, 95, 95);\n"
"padding: 3px;\n"
"color: rgb(223, 223, 223);")
        self.labelVolume.setObjectName("labelVolume")
        self.horizontalLayout_12.addWidget(self.labelVolume)
        self.layoutVolume.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.comboBoxVolume = QComboBoxIgnoreSCroll(data)
        self.comboBoxVolume.setObjectName("comboBoxVolume")
        self.horizontalLayout_2.addWidget(self.comboBoxVolume)
        self.pushButtonEditVolName = QtWidgets.QPushButton(data)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButtonEditVolName.sizePolicy().hasHeightForWidth())
        self.pushButtonEditVolName.setSizePolicy(sizePolicy)
        self.pushButtonEditVolName.setStyleSheet("border:none")
        self.pushButtonEditVolName.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/edit.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButtonEditVolName.setIcon(icon1)
        self.pushButtonEditVolName.setIconSize(QtCore.QSize(20, 20))
        self.pushButtonEditVolName.setObjectName("pushButtonEditVolName")
        self.horizontalLayout_2.addWidget(self.pushButtonEditVolName)
        self.layoutVolume.addLayout(self.horizontalLayout_2)
        self.horizontalLayoutVol1Levels = QtWidgets.QHBoxLayout()
        self.horizontalLayoutVol1Levels.setObjectName("horizontalLayoutVol1Levels")
        self.comboBoxVolumeLut = QComboBoxIgnoreSCroll(data)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxVolumeLut.sizePolicy().hasHeightForWidth())
        self.comboBoxVolumeLut.setSizePolicy(sizePolicy)
        self.comboBoxVolumeLut.setObjectName("comboBoxVolumeLut")
        self.horizontalLayoutVol1Levels.addWidget(self.comboBoxVolumeLut)
        self.layoutVolume.addLayout(self.horizontalLayoutVol1Levels)
        self.verticalLayout.addLayout(self.layoutVolume)
        self.line = QtWidgets.QFrame(data)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.vol2Widget_layout = QtWidgets.QVBoxLayout()
        self.vol2Widget_layout.setObjectName("vol2Widget_layout")
        self.volume2Label_layout = QtWidgets.QHBoxLayout()
        self.volume2Label_layout.setObjectName("volume2Label_layout")
        self.checkBoxShowVol2Controls = QtWidgets.QCheckBox(data)
        self.checkBoxShowVol2Controls.setMaximumSize(QtCore.QSize(20, 16777215))
        self.checkBoxShowVol2Controls.setText("")
        self.checkBoxShowVol2Controls.setObjectName("checkBoxShowVol2Controls")
        self.volume2Label_layout.addWidget(self.checkBoxShowVol2Controls)
        self.label_14 = QtWidgets.QLabel(data)
        self.label_14.setStyleSheet("background-color:rgb(102, 95, 95);\n"
"padding: 3px;\n"
"color: rgb(223, 223, 223);")
        self.label_14.setObjectName("label_14")
        self.volume2Label_layout.addWidget(self.label_14)
        self.checkBoxVisibilityVol2 = QtWidgets.QCheckBox(data)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkBoxVisibilityVol2.sizePolicy().hasHeightForWidth())
        self.checkBoxVisibilityVol2.setSizePolicy(sizePolicy)
        self.checkBoxVisibilityVol2.setText("")
        self.checkBoxVisibilityVol2.setChecked(True)
        self.checkBoxVisibilityVol2.setObjectName("checkBoxVisibilityVol2")
        self.volume2Label_layout.addWidget(self.checkBoxVisibilityVol2)
        self.vol2Widget_layout.addLayout(self.volume2Label_layout)
        self.vol2ControlsWidget = QtWidgets.QWidget(data)
        self.vol2ControlsWidget.setMinimumSize(QtCore.QSize(300, 80))
        self.vol2ControlsWidget.setObjectName("vol2ControlsWidget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.vol2ControlsWidget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.layoutVolume2 = QtWidgets.QVBoxLayout()
        self.layoutVolume2.setObjectName("layoutVolume2")
        self.comboBoxVolume2 = QtWidgets.QComboBox(self.vol2ControlsWidget)
        self.comboBoxVolume2.setObjectName("comboBoxVolume2")
        self.layoutVolume2.addWidget(self.comboBoxVolume2)
        self.horizontalLayoutVol2Levels = QtWidgets.QHBoxLayout()
        self.horizontalLayoutVol2Levels.setObjectName("horizontalLayoutVol2Levels")
        self.comboBoxVolumeLut2 = QtWidgets.QComboBox(self.vol2ControlsWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxVolumeLut2.sizePolicy().hasHeightForWidth())
        self.comboBoxVolumeLut2.setSizePolicy(sizePolicy)
        self.comboBoxVolumeLut2.setObjectName("comboBoxVolumeLut2")
        self.horizontalLayoutVol2Levels.addWidget(self.comboBoxVolumeLut2)
        self.doubleSpinBoxVol2Opacity = QtWidgets.QDoubleSpinBox(self.vol2ControlsWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.doubleSpinBoxVol2Opacity.sizePolicy().hasHeightForWidth())
        self.doubleSpinBoxVol2Opacity.setSizePolicy(sizePolicy)
        self.doubleSpinBoxVol2Opacity.setObjectName("doubleSpinBoxVol2Opacity")
        self.horizontalLayoutVol2Levels.addWidget(self.doubleSpinBoxVol2Opacity)
        self.label_3 = QtWidgets.QLabel(self.vol2ControlsWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setObjectName("label_3")
        self.horizontalLayoutVol2Levels.addWidget(self.label_3)
        self.layoutVolume2.addLayout(self.horizontalLayoutVol2Levels)
        self.verticalLayout_3.addLayout(self.layoutVolume2)
        self.vol2Widget_layout.addWidget(self.vol2ControlsWidget)
        self.verticalLayout.addLayout(self.vol2Widget_layout)
        self.line_2 = QtWidgets.QFrame(data)
        self.line_2.setLineWidth(5)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout.addWidget(self.line_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.checkBoxShowDataControls = QtWidgets.QCheckBox(data)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkBoxShowDataControls.sizePolicy().hasHeightForWidth())
        self.checkBoxShowDataControls.setSizePolicy(sizePolicy)
        self.checkBoxShowDataControls.setText("")
        self.checkBoxShowDataControls.setObjectName("checkBoxShowDataControls")
        self.horizontalLayout_3.addWidget(self.checkBoxShowDataControls)
        self.labelData = QtWidgets.QLabel(data)
        self.labelData.setStyleSheet("background-color:rgb(102, 95, 95);\n"
"padding: 3px;\n"
"color: rgb(223, 223, 223);")
        self.labelData.setObjectName("labelData")
        self.horizontalLayout_3.addWidget(self.labelData)
        self.checkBoxVisibilityHeatmap = QtWidgets.QCheckBox(data)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkBoxVisibilityHeatmap.sizePolicy().hasHeightForWidth())
        self.checkBoxVisibilityHeatmap.setSizePolicy(sizePolicy)
        self.checkBoxVisibilityHeatmap.setText("")
        self.checkBoxVisibilityHeatmap.setChecked(True)
        self.checkBoxVisibilityHeatmap.setObjectName("checkBoxVisibilityHeatmap")
        self.horizontalLayout_3.addWidget(self.checkBoxVisibilityHeatmap)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.dataWidget = QtWidgets.QWidget(data)
        self.dataWidget.setMinimumSize(QtCore.QSize(0, 170))
        self.dataWidget.setObjectName("dataWidget")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.dataWidget)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.comboBoxData = QtWidgets.QComboBox(self.dataWidget)
        self.comboBoxData.setObjectName("comboBoxData")
        self.verticalLayout_8.addWidget(self.comboBoxData)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.comboBoxLutHeatmap = QtWidgets.QComboBox(self.dataWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxLutHeatmap.sizePolicy().hasHeightForWidth())
        self.comboBoxLutHeatmap.setSizePolicy(sizePolicy)
        self.comboBoxLutHeatmap.setObjectName("comboBoxLutHeatmap")
        self.horizontalLayout_11.addWidget(self.comboBoxLutHeatmap)
        self.verticalLayout_8.addLayout(self.horizontalLayout_11)
        self.verticalLayout_4.addLayout(self.verticalLayout_8)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.doubleSpinBoxNegThresh = QtWidgets.QDoubleSpinBox(self.dataWidget)
        self.doubleSpinBoxNegThresh.setObjectName("doubleSpinBoxNegThresh")
        self.horizontalLayout_6.addWidget(self.doubleSpinBoxNegThresh)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem)
        self.pushButtonRecalcConnectComponents = QtWidgets.QPushButton(self.dataWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButtonRecalcConnectComponents.sizePolicy().hasHeightForWidth())
        self.pushButtonRecalcConnectComponents.setSizePolicy(sizePolicy)
        self.pushButtonRecalcConnectComponents.setObjectName("pushButtonRecalcConnectComponents")
        self.horizontalLayout_6.addWidget(self.pushButtonRecalcConnectComponents)
        self.doubleSpinBoxPosThresh = QtWidgets.QDoubleSpinBox(self.dataWidget)
        self.doubleSpinBoxPosThresh.setObjectName("doubleSpinBoxPosThresh")
        self.horizontalLayout_6.addWidget(self.doubleSpinBoxPosThresh)
        self.verticalLayout_4.addLayout(self.horizontalLayout_6)
        self.labelFdrThresholds = QtWidgets.QLabel(self.dataWidget)
        self.labelFdrThresholds.setObjectName("labelFdrThresholds")
        self.verticalLayout_4.addWidget(self.labelFdrThresholds)
        self.gridLayoutFdrButtons = QtWidgets.QGridLayout()
        self.gridLayoutFdrButtons.setObjectName("gridLayoutFdrButtons")
        self.verticalLayout_4.addLayout(self.gridLayoutFdrButtons)
        self.horizontalLayoutDataSliders = QtWidgets.QHBoxLayout()
        self.horizontalLayoutDataSliders.setObjectName("horizontalLayoutDataSliders")
        self.verticalLayout_4.addLayout(self.horizontalLayoutDataSliders)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem1)
        self.verticalLayout_4.addLayout(self.horizontalLayout_10)
        self.verticalLayoutConnectedComponents = QtWidgets.QHBoxLayout()
        self.verticalLayoutConnectedComponents.setObjectName("verticalLayoutConnectedComponents")
        self.verticalLayout_4.addLayout(self.verticalLayoutConnectedComponents)
        self.labelFdrThresholds.raise_()
        self.verticalLayout.addWidget(self.dataWidget)
        self.line_4 = QtWidgets.QFrame(data)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.verticalLayout.addWidget(self.line_4)
        self.verticalLayoutVectors = QtWidgets.QVBoxLayout()
        self.verticalLayoutVectors.setObjectName("verticalLayoutVectors")
        self.horizontalLayoutVectorHeader = QtWidgets.QHBoxLayout()
        self.horizontalLayoutVectorHeader.setObjectName("horizontalLayoutVectorHeader")
        self.checkBoxShowVectorControls = QtWidgets.QCheckBox(data)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkBoxShowVectorControls.sizePolicy().hasHeightForWidth())
        self.checkBoxShowVectorControls.setSizePolicy(sizePolicy)
        self.checkBoxShowVectorControls.setText("")
        self.checkBoxShowVectorControls.setObjectName("checkBoxShowVectorControls")
        self.horizontalLayoutVectorHeader.addWidget(self.checkBoxShowVectorControls)
        self.label_9 = QtWidgets.QLabel(data)
        self.label_9.setStyleSheet("background-color:rgb(102, 95, 95);\n"
"padding: 3px;\n"
"color: rgb(223, 223, 223);")
        self.label_9.setObjectName("label_9")
        self.horizontalLayoutVectorHeader.addWidget(self.label_9)
        self.verticalLayoutVectors.addLayout(self.horizontalLayoutVectorHeader)
        self.vectorWidget = QtWidgets.QWidget(data)
        self.vectorWidget.setMinimumSize(QtCore.QSize(0, 100))
        self.vectorWidget.setObjectName("vectorWidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.vectorWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.comboBoxVectors = QtWidgets.QComboBox(self.vectorWidget)
        self.comboBoxVectors.setObjectName("comboBoxVectors")
        self.verticalLayout_2.addWidget(self.comboBoxVectors)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_10 = QtWidgets.QLabel(self.vectorWidget)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_7.addWidget(self.label_10)
        self.spinBoxVectorSubsampling = QtWidgets.QSpinBox(self.vectorWidget)
        self.spinBoxVectorSubsampling.setObjectName("spinBoxVectorSubsampling")
        self.horizontalLayout_7.addWidget(self.spinBoxVectorSubsampling)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem2)
        self.label_11 = QtWidgets.QLabel(self.vectorWidget)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_7.addWidget(self.label_11)
        self.spinBoxVectorScale = QtWidgets.QSpinBox(self.vectorWidget)
        self.spinBoxVectorScale.setObjectName("spinBoxVectorScale")
        self.horizontalLayout_7.addWidget(self.spinBoxVectorScale)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem3)
        self.label_12 = QtWidgets.QLabel(self.vectorWidget)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_7.addWidget(self.label_12)
        self.pushButtonVectorColor = QtWidgets.QPushButton(self.vectorWidget)
        self.pushButtonVectorColor.setText("")
        self.pushButtonVectorColor.setObjectName("pushButtonVectorColor")
        self.horizontalLayout_7.addWidget(self.pushButtonVectorColor)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.pushButtonVectorMagnitudeFilter = QtWidgets.QPushButton(self.vectorWidget)
        self.pushButtonVectorMagnitudeFilter.setObjectName("pushButtonVectorMagnitudeFilter")
        self.verticalLayout_2.addWidget(self.pushButtonVectorMagnitudeFilter)
        self.horizontalLayoutMagnitudeSlider = QtWidgets.QHBoxLayout()
        self.horizontalLayoutMagnitudeSlider.setObjectName("horizontalLayoutMagnitudeSlider")
        self.verticalLayout_2.addLayout(self.horizontalLayoutMagnitudeSlider)
        self.verticalLayoutVectors.addWidget(self.vectorWidget)
        self.verticalLayout.addLayout(self.verticalLayoutVectors)
        self.line_3 = QtWidgets.QFrame(data)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout.addWidget(self.line_3)
        self.gridLayoutScalebar = QtWidgets.QGridLayout()
        self.gridLayoutScalebar.setObjectName("gridLayoutScalebar")
        self.doubleSpinBoxVoxelSize = QtWidgets.QDoubleSpinBox(data)
        self.doubleSpinBoxVoxelSize.setObjectName("doubleSpinBoxVoxelSize")
        self.gridLayoutScalebar.addWidget(self.doubleSpinBoxVoxelSize, 0, 1, 1, 1)
        self.label_15 = QtWidgets.QLabel(data)
        self.label_15.setObjectName("label_15")
        self.gridLayoutScalebar.addWidget(self.label_15, 0, 0, 1, 1)
        self.doubleSpinBoxScaleBarLength = QtWidgets.QDoubleSpinBox(data)
        self.doubleSpinBoxScaleBarLength.setObjectName("doubleSpinBoxScaleBarLength")
        self.gridLayoutScalebar.addWidget(self.doubleSpinBoxScaleBarLength, 0, 4, 1, 1)
        self.label_16 = QtWidgets.QLabel(data)
        self.label_16.setObjectName("label_16")
        self.gridLayoutScalebar.addWidget(self.label_16, 0, 3, 1, 1)
        self.checkBoxScaleBarLabel = QtWidgets.QCheckBox(data)
        self.checkBoxScaleBarLabel.setObjectName("checkBoxScaleBarLabel")
        self.gridLayoutScalebar.addWidget(self.checkBoxScaleBarLabel, 0, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayoutScalebar)
        self.line_9 = QtWidgets.QFrame(data)
        self.line_9.setLineWidth(4)
        self.line_9.setMidLineWidth(2)
        self.line_9.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_9.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_9.setObjectName("line_9")
        self.verticalLayout.addWidget(self.line_9)
        self.verticalLayoutColorScale = QtWidgets.QVBoxLayout()
        self.verticalLayoutColorScale.setObjectName("verticalLayoutColorScale")
        self.verticalLayout.addLayout(self.verticalLayoutColorScale)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem4)

        self.retranslateUi(data)
        QtCore.QMetaObject.connectSlotsByName(data)

    def retranslateUi(self, data):
        _translate = QtCore.QCoreApplication.translate
        data.setWindowTitle(_translate("data", "Form"))
        self.label_13.setText(_translate("data", "Viewport visiblity"))
        self.label.setText(_translate("data", "Edit viewport"))
        self.checkBoxLinkViews.setText(_translate("data", "Link views"))
        self.checkBox6Views.setText(_translate("data", "Two row view"))
        self.label_2.setText(_translate("data", "Orientation"))
        self.comboBoxOrientation.setItemText(0, _translate("data", "sagittal"))
        self.comboBoxOrientation.setItemText(1, _translate("data", "coronal"))
        self.comboBoxOrientation.setItemText(2, _translate("data", "axial"))
        self.comboBoxOrientation.setItemText(3, _translate("data", "Isosurface - Not implemented"))
        self.pushButtonScaleBarColor.setText(_translate("data", "scale bar color"))
        self.labelVolume.setText(_translate("data", "Volume 1"))
        self.label_14.setText(_translate("data", "Volume 2"))
        self.label_3.setText(_translate("data", "opacity"))
        self.labelData.setText(_translate("data", "Heatmap Data"))
        self.pushButtonRecalcConnectComponents.setText(_translate("data", "Find blobs"))
        self.labelFdrThresholds.setText(_translate("data", "FDR thresholds"))
        self.label_9.setText(_translate("data", "Vectors"))
        self.label_10.setText(_translate("data", "Subsample"))
        self.label_11.setText(_translate("data", "Scale"))
        self.label_12.setText(_translate("data", "Arrow colour"))
        self.pushButtonVectorMagnitudeFilter.setText(_translate("data", "filter magnitude"))
        self.label_15.setText(_translate("data", "Voxel size"))
        self.label_16.setText(_translate("data", "Scale bar length"))
        self.checkBoxScaleBarLabel.setText(_translate("data", "scale label"))
from vpv.ui.views.combobox_ignore_scroll import QComboBoxIgnoreSCroll
from . import resources_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_change_vol_name.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_VolNameDialog(object):
    def setupUi(self, VolNameDialog):
        VolNameDialog.setObjectName("VolNameDialog")
        VolNameDialog.resize(395, 94)
        self.gridLayout = QtWidgets.QGridLayout(VolNameDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.lineEditVolName = QtWidgets.QLineEdit(VolNameDialog)
        self.lineEditVolName.setObjectName("lineEditVolName")
        self.gridLayout.addWidget(self.lineEditVolName, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pushButtonOk = QtWidgets.QPushButton(VolNameDialog)
        self.pushButtonOk.setObjectName("pushButtonOk")
        self.horizontalLayout.addWidget(self.pushButtonOk)
        self.pushButtonCancel = QtWidgets.QPushButton(VolNameDialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout.addWidget(self.pushButtonCancel)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)

        self.retranslateUi(VolNameDialog)
        QtCore.QMetaObject.connectSlotsByName(VolNameDialog)

    def retranslateUi(self, VolNameDialog):
        _translate = QtCore.QCoreApplication.translate
        VolNameDialog.setWindowTitle(_translate("VolNameDialog", "Dialog"))
        self.pushButtonOk.setText(_translate("VolNameDialog", "OK"))
        self.pushButtonCancel.setText(_translate("VolNameDialog", "Cancel"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_dataviewer.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DataViewer(object):
    def setupUi(self, DataViewer):
        DataViewer.setObjectName("DataViewer")
        DataViewer.resize(430, 282)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayoutInfo = QtWidgets.QGridLayout()
        self.gridLayoutInfo.setObjectName("gridLayoutInfo")
        self.verticalLayout.addLayout(self.gridLayoutInfo)
        self.plotWidget = PlotWidget(self.dockWidgetContents)
        self.plotWidget.setObjectName("plotWidget")
        self.verticalLayout.addWidget(self.plotWidget)
        DataViewer.setWidget(self.dockWidgetContents)

        self.retranslateUi(DataViewer)
        QtCore.QMetaObject.connectSlotsByName(DataViewer)

    def retranslateUi(self, DataViewer):
        _translate = QtCore.QCoreApplication.translate
        DataViewer.setWindowTitle(_translate("DataViewer", "DockWidget"))

from pyqtgraph import PlotWidget



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_gradient_editor.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_GradientEditor(object):
    def setupUi(self, GradientEditor):
        GradientEditor.setObjectName("GradientEditor")
        GradientEditor.resize(674, 147)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(GradientEditor)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(GradientEditor)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(GradientEditor)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.horizontalLayoutMain = QtWidgets.QHBoxLayout()
        self.horizontalLayoutMain.setObjectName("horizontalLayoutMain")
        self.pushButtonCancel = QtWidgets.QPushButton(GradientEditor)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayoutMain.addWidget(self.pushButtonCancel)
        self.pushButtonOk = QtWidgets.QPushButton(GradientEditor)
        self.pushButtonOk.setObjectName("pushButtonOk")
        self.horizontalLayoutMain.addWidget(self.pushButtonOk)
        self.verticalLayout_2.addLayout(self.horizontalLayoutMain)

        self.retranslateUi(GradientEditor)
        QtCore.QMetaObject.connectSlotsByName(GradientEditor)

    def retranslateUi(self, GradientEditor):
        _translate = QtCore.QCoreApplication.translate
        GradientEditor.setWindowTitle(_translate("GradientEditor", "Dialog"))
        self.label.setText(_translate("GradientEditor", "positve LUT"))
        self.label_2.setText(_translate("GradientEditor", "negative LUT"))
        self.pushButtonCancel.setText(_translate("GradientEditor", "Cancel"))
        self.pushButtonOk.setText(_translate("GradientEditor", "OK"))




from .volume import Volume

class ImageSeriesVolume(Volume):
    """
    Contains a series of images
    """
    def __init__(self, *args):
        self.images = []
        super(ImageSeriesVolume, self).__init__(*args)
        self.levels = [float(self._arr_data.min()), float(self._arr_data.max())]

    def _load_data(self, paths, memmap=False):
        """
        :param path:
            multiple paths
        :param memmap:
        :return:
        """
        for path in paths:
            array = super(ImageSeriesVolume, self)._load_data(path, memmap)
            self.images.append(array)
        return self.images[0]

    def set_image(self, idx):
        self._arr_data = self.images[idx]

    def num_images(self):
        return len(self.images)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_annotations.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Annotations(object):
    def setupUi(self, Annotations):
        Annotations.setObjectName("Annotations")
        Annotations.resize(700, 901)
        Annotations.setMinimumSize(QtCore.QSize(0, 40))
        self.verticalLayout_17 = QtWidgets.QVBoxLayout(Annotations)
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.tabWidgetOptions = QtWidgets.QTabWidget(Annotations)
        self.tabWidgetOptions.setTabPosition(QtWidgets.QTabWidget.North)
        self.tabWidgetOptions.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.tabWidgetOptions.setObjectName("tabWidgetOptions")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.tab_3)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.comboBoxAnnotationsVolumes = QtWidgets.QComboBox(self.tab_3)
        self.comboBoxAnnotationsVolumes.setObjectName("comboBoxAnnotationsVolumes")
        self.verticalLayout.addWidget(self.comboBoxAnnotationsVolumes)
        self.line_6 = QtWidgets.QFrame(self.tab_3)
        self.line_6.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_6.setObjectName("line_6")
        self.verticalLayout.addWidget(self.line_6)
        self.horizontalLayout_20 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_20.setObjectName("horizontalLayout_20")
        self.pushButtonSaveAnnotations = QtWidgets.QPushButton(self.tab_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButtonSaveAnnotations.sizePolicy().hasHeightForWidth())
        self.pushButtonSaveAnnotations.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButtonSaveAnnotations.setIcon(icon)
        self.pushButtonSaveAnnotations.setIconSize(QtCore.QSize(32, 32))
        self.pushButtonSaveAnnotations.setObjectName("pushButtonSaveAnnotations")
        self.horizontalLayout_20.addWidget(self.pushButtonSaveAnnotations)
        self.line_5 = QtWidgets.QFrame(self.tab_3)
        self.line_5.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_5.setObjectName("line_5")
        self.horizontalLayout_20.addWidget(self.line_5)
        self.label_2 = QtWidgets.QLabel(self.tab_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_20.addWidget(self.label_2)
        self.spinBoxAnnotationCircleSize = QtWidgets.QSpinBox(self.tab_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinBoxAnnotationCircleSize.sizePolicy().hasHeightForWidth())
        self.spinBoxAnnotationCircleSize.setSizePolicy(sizePolicy)
        self.spinBoxAnnotationCircleSize.setProperty("value", 10)
        self.spinBoxAnnotationCircleSize.setObjectName("spinBoxAnnotationCircleSize")
        self.horizontalLayout_20.addWidget(self.spinBoxAnnotationCircleSize)
        self.line_7 = QtWidgets.QFrame(self.tab_3)
        self.line_7.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_7.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_7.setObjectName("line_7")
        self.horizontalLayout_20.addWidget(self.line_7)
        self.label_6 = QtWidgets.QLabel(self.tab_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_6.sizePolicy().hasHeightForWidth())
        self.label_6.setSizePolicy(sizePolicy)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_20.addWidget(self.label_6)
        self.labelXPos = QtWidgets.QLabel(self.tab_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelXPos.sizePolicy().hasHeightForWidth())
        self.labelXPos.setSizePolicy(sizePolicy)
        self.labelXPos.setMaximumSize(QtCore.QSize(20, 16777215))
        self.labelXPos.setObjectName("labelXPos")
        self.horizontalLayout_20.addWidget(self.labelXPos)
        self.label_35 = QtWidgets.QLabel(self.tab_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_35.sizePolicy().hasHeightForWidth())
        self.label_35.setSizePolicy(sizePolicy)
        self.label_35.setObjectName("label_35")
        self.horizontalLayout_20.addWidget(self.label_35)
        self.labelYPos = QtWidgets.QLabel(self.tab_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelYPos.sizePolicy().hasHeightForWidth())
        self.labelYPos.setSizePolicy(sizePolicy)
        self.labelYPos.setObjectName("labelYPos")
        self.horizontalLayout_20.addWidget(self.labelYPos)
        self.label_36 = QtWidgets.QLabel(self.tab_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_36.sizePolicy().hasHeightForWidth())
        self.label_36.setSizePolicy(sizePolicy)
        self.label_36.setObjectName("label_36")
        self.horizontalLayout_20.addWidget(self.label_36)
        self.labelZPos = QtWidgets.QLabel(self.tab_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelZPos.sizePolicy().hasHeightForWidth())
        self.labelZPos.setSizePolicy(sizePolicy)
        self.labelZPos.setObjectName("labelZPos")
        self.horizontalLayout_20.addWidget(self.labelZPos)
        self.verticalLayout.addLayout(self.horizontalLayout_20)
        self.line_3 = QtWidgets.QFrame(self.tab_3)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout.addWidget(self.line_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.line = QtWidgets.QFrame(self.tab_3)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout_4.addWidget(self.line)
        self.treeWidgetAvailableTerms = QtWidgets.QTreeWidget(self.tab_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.treeWidgetAvailableTerms.sizePolicy().hasHeightForWidth())
        self.treeWidgetAvailableTerms.setSizePolicy(sizePolicy)
        self.treeWidgetAvailableTerms.setMinimumSize(QtCore.QSize(0, 300))
        self.treeWidgetAvailableTerms.setObjectName("treeWidgetAvailableTerms")
        self.treeWidgetAvailableTerms.headerItem().setText(0, "1")
        self.horizontalLayout_4.addWidget(self.treeWidgetAvailableTerms)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.textEditAnnotationInfo = QtWidgets.QTextEdit(self.tab_3)
        self.textEditAnnotationInfo.setMaximumSize(QtCore.QSize(16777215, 100))
        self.textEditAnnotationInfo.setObjectName("textEditAnnotationInfo")
        self.verticalLayout.addWidget(self.textEditAnnotationInfo)
        self.line_2 = QtWidgets.QFrame(self.tab_3)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout.addWidget(self.line_2)
        self.horizontalLayout.addLayout(self.verticalLayout)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/pencil.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabWidgetOptions.addTab(self.tab_3, icon1, "")
        self.tab_4 = QtWidgets.QWidget()
        self.tab_4.setObjectName("tab_4")
        self.layoutWidget = QtWidgets.QWidget(self.tab_4)
        self.layoutWidget.setGeometry(QtCore.QRect(41, 31, 197, 89))
        self.layoutWidget.setObjectName("layoutWidget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.layoutWidget)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_4 = QtWidgets.QLabel(self.layoutWidget)
        self.label_4.setObjectName("label_4")
        self.gridLayout_2.addWidget(self.label_4, 0, 0, 1, 1)
        self.lineEditCentre = QtWidgets.QLineEdit(self.layoutWidget)
        self.lineEditCentre.setObjectName("lineEditCentre")
        self.gridLayout_2.addWidget(self.lineEditCentre, 0, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.layoutWidget)
        self.label_5.setObjectName("label_5")
        self.gridLayout_2.addWidget(self.label_5, 1, 0, 1, 1)
        self.lineEditStage = QtWidgets.QLineEdit(self.layoutWidget)
        self.lineEditStage.setObjectName("lineEditStage")
        self.gridLayout_2.addWidget(self.lineEditStage, 1, 1, 1, 1)
        self.lineEditModality = QtWidgets.QLineEdit(self.layoutWidget)
        self.lineEditModality.setObjectName("lineEditModality")
        self.gridLayout_2.addWidget(self.lineEditModality, 2, 1, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.layoutWidget)
        self.label_7.setObjectName("label_7")
        self.gridLayout_2.addWidget(self.label_7, 2, 0, 1, 1)
        self.layoutWidget1 = QtWidgets.QWidget(self.tab_4)
        self.layoutWidget1.setGeometry(QtCore.QRect(40, 120, 280, 59))
        self.layoutWidget1.setObjectName("layoutWidget1")
        self.gridLayout = QtWidgets.QGridLayout(self.layoutWidget1)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(self.layoutWidget1)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.lineEditAnnotatorId = QtWidgets.QLineEdit(self.layoutWidget1)
        self.lineEditAnnotatorId.setObjectName("lineEditAnnotatorId")
        self.gridLayout.addWidget(self.lineEditAnnotatorId, 0, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.layoutWidget1)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.dateEdit = QtWidgets.QDateEdit(self.layoutWidget1)
        self.dateEdit.setObjectName("dateEdit")
        self.gridLayout.addWidget(self.dateEdit, 1, 1, 1, 1)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/settings.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabWidgetOptions.addTab(self.tab_4, icon2, "")
        self.verticalLayout_17.addWidget(self.tabWidgetOptions)

        self.retranslateUi(Annotations)
        self.tabWidgetOptions.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(Annotations)

    def retranslateUi(self, Annotations):
        _translate = QtCore.QCoreApplication.translate
        Annotations.setWindowTitle(_translate("Annotations", "Form"))
        self.pushButtonSaveAnnotations.setToolTip(_translate("Annotations", "Save annotation as xml"))
        self.pushButtonSaveAnnotations.setText(_translate("Annotations", "Save annotations"))
        self.label_2.setText(_translate("Annotations", "Circle radius"))
        self.spinBoxAnnotationCircleSize.setToolTip(_translate("Annotations", "radius of the marker"))
        self.label_6.setText(_translate("Annotations", "x:"))
        self.labelXPos.setText(_translate("Annotations", "0"))
        self.label_35.setText(_translate("Annotations", "Y:"))
        self.labelYPos.setText(_translate("Annotations", "0"))
        self.label_36.setText(_translate("Annotations", "Z:"))
        self.labelZPos.setText(_translate("Annotations", "0"))
        self.label_4.setText(_translate("Annotations", "Centre"))
        self.label_5.setText(_translate("Annotations", "Stage"))
        self.label_7.setText(_translate("Annotations", "Modality"))
        self.label.setText(_translate("Annotations", "Annotator ID"))
        self.label_3.setText(_translate("Annotations", "Date of annotation"))


from . import resources_rc



# Copyright 2016 Medical Research Council Harwell.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# @author Neil Horner <n.horner@har.mrc.ac.uk>

import appdirs
import yaml
from os.path import expanduser
import os
import collections
from vpv import common
import logging

VPV_APPDATA_VERSION = 2.2
ANNOTATION_CRICLE_RADIUS_DEFAULT = 40


class AppData(object):
    def __init__(self):
        self.using_appdata = True # Set to false if we weren't able to find a sirectory to save the appdata
        appname = 'vpv_viewer'
        appdata_dir = appdirs.user_data_dir(appname)

        if not os.path.isdir(appdata_dir):
            try:
                os.mkdir(appdata_dir)
            except WindowsError:
                #Can't find the AppData directory. So just make one in home directory
                appdata_dir = os.path.join(expanduser("~"), '.' + appname)
                if not os.path.isdir(appdata_dir):
                    os.mkdir(appdata_dir)

        self.app_data_file = os.path.join(appdata_dir, 'vpv_viewer.yaml')

        if os.path.isfile(self.app_data_file):

            self.monster_munch = common.load_yaml(self.app_data_file)
            if not self.monster_munch:
                logging.error(f'Warning: could not load app data file\nTry deleting {self.app_data_file}')
                self.monster_munch = {}
        else:
            self.monster_munch = {}

        # Appdata versioning was not always in place. If a we find some appdata without a version, reset the data
        # Also reset the data if we find a previous version

        # This breaks for JIm on v2.02. Complains about NoneType not having a get(). Catch attribute error
        try:
            if self.monster_munch.get('version') is None or self.monster_munch['version'] < VPV_APPDATA_VERSION:
                print("resetting appdata")
                self.monster_munch = {}
        except AttributeError:
            self.monster_munch = {}

        if self.monster_munch == {}:
            self.monster_munch['version'] = VPV_APPDATA_VERSION

        if 'recent_files' not in self.monster_munch:
            self.monster_munch['recent_files'] = collections.deque(maxlen=10)

    def set_flips(self, flip_options: dict):
        self.monster_munch['flips'] = flip_options

    def get_flips(self):
        flips = self.monster_munch.get('flips')

        if not flips:
            self.monster_munch['flips'] = {'axial':    {'x': False, 'z': False, 'y': False},
                                      'coronal':  {'x': False, 'z': False, 'y': False},
                                      'sagittal': {'x': False, 'z': False, 'y': False},
                                      'impc_view': False}

        return self.monster_munch['flips']

    @property
    def annotation_circle_radius(self):
        return self.monster_munch.get('annotation_cricle_radius', ANNOTATION_CRICLE_RADIUS_DEFAULT)

    @annotation_circle_radius.setter
    def annotation_circle_radius(self, radius):
        self.monster_munch['annotation_cricle_radius'] = radius

    @property
    def annotation_centre(self):
        return self.monster_munch.get('annotation_centre')

    @annotation_centre.setter
    def annotation_centre(self, centre):
        self.monster_munch['annotation_centre'] = centre

    @property
    def annotation_stage(self):
        return self.monster_munch.get('annotation_stage')

    @annotation_stage.setter
    def annotation_stage(self, stage):
       self.monster_munch['annotation_stage'] = stage

    @property
    def annotator_id(self):
        return self.monster_munch.get('annotator_id')

    @annotator_id.setter
    def annotator_id(self, id_):
        self.monster_munch['annotator_id'] = id_

    @property
    def last_screen_shot_dir(self):
        if not self.monster_munch.get('last_screenshot_dir'):
            self.monster_munch['last_screenshot_dir'] = expanduser("~")
        return self.monster_munch['last_screenshot_dir']

    @last_screen_shot_dir.setter
    def last_screen_shot_dir(self, dir_):
        self.monster_munch['last_screenshot_dir'] = dir_

    def write_app_data(self):
        #first convert yaml-incompatible stuff
        self.monster_munch['recent_files'] = list(self.monster_munch['recent_files'])[0: 10]
        with open(self.app_data_file, 'w') as fh:
            fh.write(yaml.dump(self.monster_munch))

    def get_last_dir_browsed(self):
        if not self.monster_munch.get('last_dir_browsed'):
            self.monster_munch['last_dir_browsed'] = expanduser("~")
        return self.monster_munch['last_dir_browsed']

    def set_last_dir_browsed(self, path):
        self.monster_munch['last_dir_browsed'] = path

    def add_used_volume(self, volpath):
        if volpath not in self.monster_munch['recent_files']:
            self.monster_munch['recent_files'].append(volpath)

    def get_recent_files(self):
        return self.monster_munch['recent_files']

    def set_include_filter_patterns(self, patterns):
        self.monster_munch['include_patterns'] = patterns

    def get_include_filter_patterns(self):
        if not self.monster_munch.get('include_patterns'):
            return []
        else:
            return self.monster_munch.get('include_patterns')

    def set_exclude_filter_patterns(self, patterns):
        self.monster_munch['exclude_patterns'] = patterns

    def get_exclude_filter_patterns(self):
        if not self.monster_munch.get('exclude_patterns'):
            return []
        else:
            return self.monster_munch.get('exclude_patterns')

    def clear_recent(self):
        self.monster_munch['recent_files'].clear()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_sliceLabels.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(526, 81)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.labelVolume = QtWidgets.QLabel(Form)
        self.labelVolume.setObjectName("labelVolume")
        self.verticalLayout.addWidget(self.labelVolume)
        self.labelData = QtWidgets.QLabel(Form)
        self.labelData.setObjectName("labelData")
        self.verticalLayout.addWidget(self.labelData)
        self.labelVectors = QtWidgets.QLabel(Form)
        self.labelVectors.setObjectName("labelVectors")
        self.verticalLayout.addWidget(self.labelVectors)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.labelVolume.setText(_translate("Form", "TextLabel"))
        self.labelData.setText(_translate("Form", "TextLabel"))
        self.labelVectors.setText(_translate("Form", "TextLabel"))




# Copyright 2016 Medical Research Council Harwell.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# @author Neil Horner <n.horner@har.mrc.ac.uk>

from PyQt5.QtWidgets import QDockWidget
from vpv.utils.lookup_tables import Lut
from vpv.ui.views.ui_manager import Ui_ManageViews


class ManagerDockWidget(QDockWidget):

    def __init__(self, model, mainwindow, appdata, data_manager, annotations_manager, options, console):
        super(ManagerDockWidget, self).__init__(mainwindow)
        lut = Lut()
        self.appdata = appdata
        self.data_manager = data_manager
        self.annotations = annotations_manager
        self.options_tab = options
        self.console = console
        self.tab_map = {0: self.data_manager,
                        1: self.annotations}
        if self.console:
            self.tab_map[2] = self.console
        self.tab_map[3] = self.options_tab
        self.hotred = lut._hot_red_blue()[0]
        self.hotblue = lut._hot_red_blue()[1]
        self.ui = Ui_ManageViews()
        self.ui.setupUi(self)
        self.setStyleSheet("font-size: 12px")
        self.setFeatures(QDockWidget.AllDockWidgetFeatures)
        self.model = model
        self.mainwindow = mainwindow
        self.volume_ids = None
        self.luts = Lut()

        self.current_view_id = 0
        self.current_view = None

        self.link_views = True

        self.ui.tabWidget.addTab(self.data_manager, 'Data')

        self.annotations.annotation_recent_dir_signal.connect(self.appdata.set_last_dir_browsed)
        self.ui.tabWidget.addTab(self.annotations, 'Annotations')
        self.ui.tabWidget.currentChanged.connect(self.tab_changed)

        self.ui.tabWidget.addTab(self.options_tab, 'Options')

        if self.console:
            self.ui.tabWidget.addTab(self.console, 'Console')

    def tab_changed(self, indx):
        """
        When changing tab, execute code required for that tab
        """
        if indx == 1:
            self.tab_map.get(indx).activate_tab()
            if indx == 1:  # Need to link views for the annotations tab
                self.data_manager.link_views = True
                self.annotations.annotating = True
            else:
                self.annotations.annotating = False
        else:
            self.annotations.annotating = False

    def switch_tab(self, idx):
        self.ui.tabWidget.setCurrentIndex(idx)

    # def mouse_pressed(self, view_index, x, y, orientation, vol_id):   # delete
    #     # Only do annotations when annotation tab is visible
    #     # if self.ui.tabWidget.isTabEnabled(2):
    #     self.annotations.mouse_pressed_annotate(view_index, x, y, orientation, vol_id)

    def volume_changed(self, vol_name):
        """
        When volume is changed from the combobox
        """
        self.data.modify_layer(0, 'set_volume', str(vol_name))

    def volume2_changed(self, vol_name):
        """
        When volume is changed from the combobox
        """
        self.data.modify_layer(1, 'set_volume', str(vol_name))

    def update(self):

        self.mainwindow.manager_layout.addWidget(self, 0)
        self.show()

        self.data.update()
        self.annotations.update()



from .volume import Volume
from vpv.annotations.annotations_model import SpecimenAnnotations


class ImageVolume(Volume):
    def __init__(self, *args):
        super(ImageVolume, self).__init__(*args)

        # We have annotations only on ImageVolumes
        self.annotations = SpecimenAnnotations(self.shape_xyz(), self.vol_path)
        self.levels = [float(self._arr_data.min()), float(self._arr_data.max())]




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_loading_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Loading(object):
    def setupUi(self, Loading):
        Loading.setObjectName("Loading")
        Loading.resize(501, 133)
        self.gridLayout = QtWidgets.QGridLayout(Loading)
        self.gridLayout.setObjectName("gridLayout")
        self.labelMessage1 = QtWidgets.QLabel(Loading)
        self.labelMessage1.setText("")
        self.labelMessage1.setObjectName("labelMessage1")
        self.gridLayout.addWidget(self.labelMessage1, 0, 0, 1, 1)
        self.labelMessage2 = QtWidgets.QLabel(Loading)
        self.labelMessage2.setText("")
        self.labelMessage2.setObjectName("labelMessage2")
        self.gridLayout.addWidget(self.labelMessage2, 1, 0, 1, 1)

        self.retranslateUi(Loading)
        QtCore.QMetaObject.connectSlotsByName(Loading)

    def retranslateUi(self, Loading):
        _translate = QtCore.QCoreApplication.translate
        Loading.setWindowTitle(_translate("Loading", "Dialog"))




from PyQt5.QtWidgets import QScrollArea, QWidget, QVBoxLayout, QLabel


class Window(QScrollArea):
    def __init__(self):
        super(Window, self).__init__()
        widget = QWidget()
        layout = QVBoxLayout(widget)
        # layout.setAlignment(Qt.AlignTop)
        for index in range(100):
            layout.addWidget(QLabel('Label %02d' % index))
        self.setWidget(widget)
        self.setWidgetResizable(True)


from vpv.common import Orientation, read_image
import numpy as np


class VectorVolume(object):
    """
    Holds deformation vector field volume.
    Does not inherit from Volume as it's so different
    """
    def __init__(self,  vol, model, datatype):
        self.model = model
        self.datatype = datatype
        self._arr_data = self._load_data(vol)
        self.shape = self._arr_data.shape
        self.scale = 1
        self.subsampling = 5

    def _load_data(self, vol, memap=False):
        return read_image(vol)

    def get_coronal(self, index):
        #slice_ = np.rot90(self._arr_data[:, index, :], 1)
        slice_ = np.fliplr(self._arr_data[:, index, :])
        return slice_

    def get_sagittal(self, index):
        slice_ = self._arr_data[:, :, index]
        #slice_ = np.rot90(self._arr_data[:, :, index], 1)
        return slice_

    def get_axial(self, index):
        slice_ = np.flipud(self._arr_data[index, :, :])
        return slice_

    def dimension_length(self, orientation):
        """
        Temp bodge. return the number of slices in this dimension
        :param orientation:
        :return:
        """
        if orientation == Orientation.sagittal:
            return self._arr_data[0, 0, :, 0].size
        if orientation == Orientation.coronal:
            return self._arr_data[0, :, 0, 0].size
        if orientation == Orientation.axial:
            return self._arr_data[:, 0, 0, 0].size


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_importer.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(967, 507)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Dialog.sizePolicy().hasHeightForWidth())
        Dialog.setSizePolicy(sizePolicy)
        Dialog.setMinimumSize(QtCore.QSize(500, 130))
        self.gridLayout_2 = QtWidgets.QGridLayout(Dialog)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.table = QtWidgets.QTableWidget(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.table.sizePolicy().hasHeightForWidth())
        self.table.setSizePolicy(sizePolicy)
        self.table.setLineWidth(2)
        self.table.setMidLineWidth(1)
        self.table.setColumnCount(3)
        self.table.setObjectName("table")
        self.table.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(2, item)
        self.gridLayout_2.addWidget(self.table, 4, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pushButtonChooseFile = QtWidgets.QPushButton(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButtonChooseFile.sizePolicy().hasHeightForWidth())
        self.pushButtonChooseFile.setSizePolicy(sizePolicy)
        self.pushButtonChooseFile.setObjectName("pushButtonChooseFile")
        self.horizontalLayout.addWidget(self.pushButtonChooseFile)
        self.pushButtonChoseDir = QtWidgets.QPushButton(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButtonChoseDir.sizePolicy().hasHeightForWidth())
        self.pushButtonChoseDir.setSizePolicy(sizePolicy)
        self.pushButtonChoseDir.setObjectName("pushButtonChoseDir")
        self.horizontalLayout.addWidget(self.pushButtonChoseDir)
        self.checkBoxMemoryMap = QtWidgets.QCheckBox(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkBoxMemoryMap.sizePolicy().hasHeightForWidth())
        self.checkBoxMemoryMap.setSizePolicy(sizePolicy)
        self.checkBoxMemoryMap.setChecked(False)
        self.checkBoxMemoryMap.setObjectName("checkBoxMemoryMap")
        self.horizontalLayout.addWidget(self.checkBoxMemoryMap)
        self.checkBoxAllSameType = QtWidgets.QCheckBox(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkBoxAllSameType.sizePolicy().hasHeightForWidth())
        self.checkBoxAllSameType.setSizePolicy(sizePolicy)
        self.checkBoxAllSameType.setObjectName("checkBoxAllSameType")
        self.horizontalLayout.addWidget(self.checkBoxAllSameType)
        self.checkBoxDistrbute = QtWidgets.QCheckBox(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkBoxDistrbute.sizePolicy().hasHeightForWidth())
        self.checkBoxDistrbute.setSizePolicy(sizePolicy)
        self.checkBoxDistrbute.setObjectName("checkBoxDistrbute")
        self.horizontalLayout.addWidget(self.checkBoxDistrbute)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.gridLayout_2.addLayout(self.horizontalLayout, 0, 0, 1, 1)
        self.verticalLayoutVirtualStack = QtWidgets.QVBoxLayout()
        self.verticalLayoutVirtualStack.setObjectName("verticalLayoutVirtualStack")
        self.virtualStackWidget = QtWidgets.QWidget(Dialog)
        self.virtualStackWidget.setMinimumSize(QtCore.QSize(0, 130))
        self.virtualStackWidget.setObjectName("virtualStackWidget")
        self.gridLayout = QtWidgets.QGridLayout(self.virtualStackWidget)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(self.virtualStackWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.lineEditFileNameContains = QtWidgets.QLineEdit(self.virtualStackWidget)
        self.lineEditFileNameContains.setObjectName("lineEditFileNameContains")
        self.gridLayout.addWidget(self.lineEditFileNameContains, 0, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.virtualStackWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.lineEditFileNameDoesNotContain = QtWidgets.QLineEdit(self.virtualStackWidget)
        self.lineEditFileNameDoesNotContain.setObjectName("lineEditFileNameDoesNotContain")
        self.gridLayout.addWidget(self.lineEditFileNameDoesNotContain, 1, 1, 1, 1)
        self.pushButtonFilter = QtWidgets.QPushButton(self.virtualStackWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButtonFilter.sizePolicy().hasHeightForWidth())
        self.pushButtonFilter.setSizePolicy(sizePolicy)
        self.pushButtonFilter.setObjectName("pushButtonFilter")
        self.gridLayout.addWidget(self.pushButtonFilter, 3, 0, 1, 1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_4 = QtWidgets.QLabel(self.virtualStackWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_3.addWidget(self.label_4)
        self.labelNumberOfFiles = QtWidgets.QLabel(self.virtualStackWidget)
        self.labelNumberOfFiles.setText("")
        self.labelNumberOfFiles.setObjectName("labelNumberOfFiles")
        self.horizontalLayout_3.addWidget(self.labelNumberOfFiles)
        self.gridLayout.addLayout(self.horizontalLayout_3, 3, 1, 1, 1)
        self.verticalLayoutVirtualStack.addWidget(self.virtualStackWidget)
        self.gridLayout_2.addLayout(self.verticalLayoutVirtualStack, 2, 0, 1, 1)
        self.line_2 = QtWidgets.QFrame(Dialog)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.gridLayout_2.addWidget(self.line_2, 3, 0, 1, 1)
        self.line = QtWidgets.QFrame(Dialog)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_2.addWidget(self.line, 5, 0, 1, 1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.pushButtonOK = QtWidgets.QPushButton(Dialog)
        self.pushButtonOK.setObjectName("pushButtonOK")
        self.horizontalLayout_2.addWidget(self.pushButtonOK)
        self.pushButtonCancel = QtWidgets.QPushButton(Dialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout_2.addWidget(self.pushButtonCancel)
        self.gridLayout_2.addLayout(self.horizontalLayout_2, 6, 0, 1, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_3 = QtWidgets.QLabel(Dialog)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_4.addWidget(self.label_3)
        self.lineEditFolderFilter = QtWidgets.QLineEdit(Dialog)
        self.lineEditFolderFilter.setObjectName("lineEditFolderFilter")
        self.horizontalLayout_4.addWidget(self.lineEditFolderFilter)
        self.gridLayout_2.addLayout(self.horizontalLayout_4, 1, 0, 1, 1)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        item = self.table.horizontalHeaderItem(0)
        item.setText(_translate("Dialog", "Data"))
        item = self.table.horizontalHeaderItem(1)
        item.setText(_translate("Dialog", "Type"))
        item = self.table.horizontalHeaderItem(2)
        item.setText(_translate("Dialog", "Load?"))
        self.pushButtonChooseFile.setText(_translate("Dialog", "Choose files"))
        self.pushButtonChoseDir.setText(_translate("Dialog", "Load stack of 2D slices"))
        self.checkBoxMemoryMap.setText(_translate("Dialog", "memory map volumes"))
        self.checkBoxAllSameType.setText(_translate("Dialog", "All same type?"))
        self.checkBoxDistrbute.setText(_translate("Dialog", "distribute across views"))
        self.label.setText(_translate("Dialog", "Filename contains (eg:.   .tiff)"))
        self.label_2.setText(_translate("Dialog", "Filename does not contain (eg: spr.tif, rec.tif)"))
        self.pushButtonFilter.setText(_translate("Dialog", "Filter"))
        self.label_4.setText(_translate("Dialog", "Number of files:"))
        self.pushButtonOK.setText(_translate("Dialog", "OK"))
        self.pushButtonCancel.setText(_translate("Dialog", "Cancel"))
        self.label_3.setToolTip(_translate("Dialog", "<html><head/><body><p>Filter volumes using wildcard patterns..</p><p><br/></p><p>For example if you have data structure like this</p><p><br/></p><p>root_dir</p><p>    - mutant_1</p><p>        - registration</p><p>             - images</p><p>             - deformations</p><p>        - other</p><p>    - mutant_2</p><p>        - registration</p><p>             - images</p><p>             - deformations</p><p>        - other</p><p>Using the following pattern would load volumes from the registration/images subfolders only</p><p><br/></p><p><span style=\" font-style:italic;\">*reg*image*</span></p></body></html>"))
        self.label_3.setText(_translate("Dialog", "filter"))
        self.lineEditFolderFilter.setToolTip(_translate("Dialog", "<html><head/><body><p>Enter a search pattern (wuth wild cards *) to filter input images by.</p><p>For example, using pattern \'reg*rig\' would find all images that had prent folderds named registrations/rigid</p></body></html>"))





import numpy as np
from PyQt5 import QtGui, QtCore
from PyQt5.QtWidgets import QDialog
import pyqtgraph as pg
from vpv.lib.qrangeslider import QRangeSlider
from vpv.utils.lookup_tables import Lut
from vpv.ui.views.ui_datatab import Ui_data
from vpv.ui.views.ui_change_vol_name import Ui_VolNameDialog
import copy
from vpv.common import Orientation, Layers
from functools import partial

"""
The Qt widget that controls the currently viewed volumes, heatmaps, and vector volumes. Accessed from the main 
dock widget
"""

DEFAULT_SCALE_BAR_SIZE = 14.0


class VolNameDialog(QDialog):
    name_changed_signal = QtCore.pyqtSignal(str, str)

    def __init__(self, parent, current_name):
        super(VolNameDialog, self).__init__(parent)
        self.current_name = copy.copy(current_name)
        self.ui = Ui_VolNameDialog()
        self.ui.setupUi(self)
        self.setWindowTitle('Edit volume name')
        self.ui.lineEditVolName.setText(current_name)
        self.ui.pushButtonOk.clicked.connect(self.on_ok)
        self.ui.pushButtonCancel.clicked.connect(self.on_cancel)

    def on_ok(self):
        new_name = self.ui.lineEditVolName.text()
        self.name_changed_signal.emit(self.current_name, new_name)
        self.close()

    def on_cancel(self):
        self.close()


class ManageData(QtGui.QWidget):
    data_processing_signal = QtCore.pyqtSignal()
    data_processing_finished_signal = QtCore.pyqtSignal()
    roi_signal = QtCore.pyqtSignal(list, list, list)
    ui_changed_signal = QtCore.pyqtSignal()
    scale_bar_color_signal = QtCore.pyqtSignal(QtGui.QColor)
    gradient_editor_signal = QtCore.pyqtSignal()

    def __init__(self, controller, model, mainwindow, appdata):
        super(ManageData, self).__init__(mainwindow)
        self.ui = Ui_data()
        self.ui.setupUi(self)
        self.views = controller.views
        self.mainwindow = mainwindow
        self.appdata = appdata
        lut = Lut()
        self.controller = controller  # run_vpv.py

        self.hotred = lut._hot_red_blue()[0]
        self.hotblue = lut._hot_red_blue()[1]

        self.model = model
        self.volume_ids = None
        self.luts = Lut()

        self.ui.labelFdrThresholds.hide()

        self._link_views = True
        self.ui.checkBoxLinkViews.setChecked(True)

        self.vector_mag_slider = QRangeSlider((0, 0, 0))
        # Just need one handle, so hide the minimum handle
        self.vector_mag_slider.head.setDisabled(True)
        self.ui.horizontalLayoutMagnitudeSlider.insertWidget(1, self.vector_mag_slider)

        self.vector_mag_slider.setMin(0)
        self.vector_mag_slider.setMax(10.0)
        self.vector_mag_slider.setStart(0)
        self.vector_mag_slider.setEnd(10.0)

        # The lower volume levels slider and Combo boxes
        self.volume_levels_slider = QRangeSlider((255, 255, 255))
        self.ui.horizontalLayoutVol1Levels.insertWidget(1, self.volume_levels_slider)

        # upper volume levels slider and comboboxes
        self.volume_levels_slider2 = QRangeSlider((255, 255, 255))
        self.ui.horizontalLayoutVol2Levels.insertWidget(1, self.volume_levels_slider2)

        # data sliders
        self.data_levels_positive_slider = QRangeSlider((0, 255, 0))
        pos_bg = 'background: qlineargradient(x1: 0.2, x2: 1,stop: 0 #FFFFFF, stop: 1  #FF0000);'
        self.data_levels_positive_slider.handle.setStyleSheet(pos_bg)
        self.ui.horizontalLayoutDataSliders.insertWidget(1, self.data_levels_positive_slider)

        self.data_levels_negative_slider = QRangeSlider((0, 255, 0))
        neg_bg = 'background: qlineargradient(x1: 0.2, x2: 1,stop: 0 #0000FF, stop: 1  #FFFFFF);'
        self.data_levels_negative_slider.handle.setStyleSheet(neg_bg)
        self.ui.horizontalLayoutDataSliders.insertWidget(0, self.data_levels_negative_slider)

        self.ui.pushButtonManagerGrey.hide()
        self.ui.pushButtonManagerCyan.hide()
        self.ui.pushButtonManagerOrange.hide()

        self.six_views_visible = False

        self.ui.checkBox6Views.setChecked(False)

        self.blob_table = QtGui.QTableWidget(self)
        self.ui.verticalLayoutConnectedComponents.addWidget(self.blob_table, 0)

        self.blob_table.setColumnCount(3)
        self.blob_table.setHorizontalHeaderLabels(['Count', 'Mean', 'location'])

        self.ui.doubleSpinBoxVoxelSize.setMaximum(1000.0)
        self.ui.doubleSpinBoxVoxelSize.setValue(DEFAULT_SCALE_BAR_SIZE)
        self.ui.doubleSpinBoxVoxelSize.setKeyboardTracking(False)
        self.ui.doubleSpinBoxVoxelSize.valueChanged.connect(self.set_voxel_size)

        self.ui.doubleSpinBoxScaleBarLength.setMaximum(10000)
        self.ui.doubleSpinBoxScaleBarLength.setValue(1000)
        self.ui.doubleSpinBoxScaleBarLength.setKeyboardTracking(False)

        self.colour_bar = ColorScaleBar(self.ui.verticalLayoutColorScale, self.hotblue, self.hotred)
        self.ui.vol2ControlsWidget.hide()
        self.ui.vectorWidget.hide()
        self.ui.dataWidget.hide()

        self.ui.doubleSpinBoxNegThresh.setMaximum(0)
        self.ui.doubleSpinBoxNegThresh.setMinimum(-100)
        self.ui.doubleSpinBoxNegThresh.setSingleStep(0.1)

        self.ui.doubleSpinBoxPosThresh.setSingleStep(0.1)

        self.ui.doubleSpinBoxVol2Opacity.setRange(0.0, 1.0)
        self.ui.doubleSpinBoxVol2Opacity.setSingleStep(0.1)
        self.ui.doubleSpinBoxVol2Opacity.setValue(1.0)

        self.connect_signal_slots()

    def connect_signal_slots(self):
        self.ui.pushButtonRecalcConnectComponents.clicked.connect(self.controller.recalc_connected_components)
        self.ui.comboBoxVolume.activated['QString'].connect(partial(self.modify_layer, Layers.vol1, 'set_volume'))

        self.ui.checkBoxVisibilityVol2.clicked.connect(partial(self.modify_layer, Layers.vol2, 'set_visibility'))
        self.ui.checkBoxVisibilityHeatmap.clicked.connect(partial(self.modify_layer, Layers.heatmap, 'set_visibility'))

        self.ui.checkBoxLinkViews.clicked.connect(self.on_link_views)

        self.ui.comboBoxOrientation.activated['QString'].connect(self.on_orientation)

        self.ui.pushButtonScreenShot.clicked.connect(self.controller.take_screen_shot)

        self.ui.pushButtonVectorMagnitudeFilter.pressed.connect(self.lower_magnitude_changed)

        self.volume_levels_slider.startValueChanged.connect(self.lower_level_volume_changed)
        self.volume_levels_slider.endValueChanged.connect(self.upper_level_volume_changed)
        self.ui.comboBoxVolumeLut.activated['QString'].connect(self.on_vol_lut_changed)
        self.volume_levels_slider2.startValueChanged.connect(self.lower_level_volume2_changed)
        self.volume_levels_slider2.endValueChanged.connect(self.upper_level_volume2_changed)

        self.ui.comboBoxVolume2.activated['QString'].connect(self.volume2_changed)
        self.ui.comboBoxVolumeLut2.activated['QString'].connect(self.on_vol2_lut_changed)
        self.ui.comboBoxLutHeatmap.activated['QString'].connect(self.on_heatmap_lut_changed)

        self.ui.comboBoxData.activated['QString'].connect(self.data_changed)

        # connect the levels slider to the model
        self.data_levels_negative_slider.startValueChanged.connect(self.data_negative_lower_changed)
        self.data_levels_negative_slider.endValueChanged.connect(self.data_negative_higher_changed)

        self.data_levels_positive_slider.startValueChanged.connect(self.data_positive_lower_changed)
        self.data_levels_positive_slider.endValueChanged.connect(self.data_positive_higher_changed)

        self.ui.pushButtonManagerRed.clicked.connect(self.showRedViewManagerSlot)
        self.ui.pushButtonManagerBlue.clicked.connect(self.showBlueViewManagerSlot)
        self.ui.pushButtonManagerGreen.clicked.connect(self.showGreenViewManagerSlot)
        self.ui.pushButtonManagerOrange.clicked.connect(self.showOrangeViewManagerSlot)
        self.ui.pushButtonManagerGrey.clicked.connect(self.showYellowViewManagerSlot)
        self.ui.pushButtonManagerCyan.clicked.connect(self.showCyanViewManagerSlot)

        self.ui.checkBoxLeftView.clicked.connect(self.left_view_visibility)
        self.ui.checkBoxCentralView.clicked.connect(self.central_view_visibility)
        self.ui.checkBoxRightView.clicked.connect(self.right_view_visibility)
        # connect the vector controls
        self.ui.comboBoxVectors.activated['QString'].connect(self.vector_changed)
        self.ui.spinBoxVectorScale.valueChanged['QString'].connect(self.vector_scale_changed)
        self.ui.spinBoxVectorSubsampling.valueChanged.connect(self.vector_subsampling_changed)
        self.ui.pushButtonVectorColor.pressed.connect(self.vector_change_color)

        self.blob_table.cellClicked.connect(self.on_connected_table_clicked)
        self.ui.doubleSpinBoxScaleBarLength.valueChanged.connect(self.set_scalebar_length)

        self.ui.checkBoxShowVol2Controls.clicked.connect(self.vol2_controls_visibility)
        self.ui.checkBoxShowVectorControls.clicked.connect(self.vector_controls_visibility)
        self.ui.checkBoxShowDataControls.clicked.connect(self.data_controls_visibility)

        self.ui.pushButtonEditVolName.clicked.connect(self.on_change_vol_name)

        self.ui.pushButtonScaleBarColor.clicked.connect(self.on_scalebar_color)

        self.ui.doubleSpinBoxNegThresh.valueChanged.connect(self.on_neg_thresh_spin)

        self.ui.checkBox6Views.clicked.connect(self.show2Rows)

        self.ui.doubleSpinBoxPosThresh.valueChanged.connect(self.on_pos_thresh_spin)
        self.ui.checkBoxScaleBarLabel.clicked.connect(self.on_scalebar_label_checked)

        self.ui.doubleSpinBoxVol2Opacity.valueChanged.connect(partial(self.modify_layer, Layers.vol2, 'set_opacity'))

    def on_scalebar_label_checked(self, checked):
        for view in self.views.values():
            view.scale_bar_visible = checked

    def activate_tab(self):
            pass

    def on_neg_thresh_spin(self, value):
        self.data_levels_negative_slider.setEnd(value)

    def on_pos_thresh_spin(self, value):
        self.data_levels_positive_slider.setStart(value)

    def on_pos_lut(self):
        print('gradient in dm')
        self.gradient_editor_signal.emit() # called 5/6 times on one click

    def on_scalebar_color(self):
        color = QtGui.QColorDialog.getColor()
        self.scale_bar_color_signal.emit(color)

    def on_change_vol_name(self):
        vol = self.controller.current_view.layers[Layers.vol1].vol
        if vol:
            dlg = VolNameDialog(self, vol.name)
            dlg.name_changed_signal.connect(self.change_vol_name)
            dlg.show()

    def change_vol_name(self, current_name, new_name):
        vol = self.controller.current_view.layers[Layers.vol1].vol

        if vol and new_name:
            self.controller.model.change_vol_name(current_name, new_name)
            self.controller.update_slice_views()
            self.update()

    # def mouse_pressed(self, view_index, x, y, orientation, vol_id):
    #     self.annotations.mouse_pressed_annotate(view_index, x, y, orientation, vol_id)

    def data_controls_visibility(self, checked):
        if checked:
            self.ui.dataWidget.show()
        else:
            self.ui.dataWidget.hide()

    def vector_controls_visibility(self, checked):
        if checked:
            self.ui.vectorWidget.show()
        else:
            self.ui.vectorWidget.hide()

    def vol2_controls_visibility(self, checked):
        if checked:
            self.ui.vol2ControlsWidget.show()
        else:
            self.ui.vol2ControlsWidget.hide()

    def show_color_scale_bars(self, isvisible=True):
        """
        Make color scale bar figure. A bit of a mess at the moment. Will tody up
        """
        if not self.controller.current_view.layers[Layers.heatmap].vol:
            return
        if not isvisible:
            self.colour_bar.hide()
            return
        else:
            self.colour_bar.show()

    def set_scalebar_length(self, length):
        for view in self.views.values():
            view.set_scalebar_size(length)

    def set_voxel_size(self, voxel_size):
        for view in self.views.values():
            view.set_voxel_size(voxel_size)

    def lower_magnitude_changed(self):
        value = self.vector_mag_slider.getRange()[0]
        if not self.link_views:
            self.current_slice_view.layers[Layers.vectors].set_magnitude_cutoff(value, 10.0)
        else:
            for view in self.views.values():
                view.layers[Layers.heatmap].set_magnitude_cutoff(value, 10.0)
        #self.update_slice_views()

    def vector_subsampling_changed(self, value):
        if self.link_views:
            for view in self.views.values():
                view.layers[Layers.vectors].set_subsampling(value)
        else:
            self.controller.current_view.layers[Layers.vectors].set_subsampling(value)

    def vector_scale_changed(self, value):
        if self.link_views:
            for view in self.views.values():
                view.layers[Layers.vectors].set_scale(value)
        else:
            self.controller.current_view.layers[Layers.vectors].set_scale(value)

    def vector_change_color(self):
        col = QtGui.QColorDialog.getColor()

        if col.isValid():
            self.modify_layer(Layers.vectors, 'set_arrow_color', col)

    def showRedViewManagerSlot(self):
        self.switch_selected_view(0)

    def showBlueViewManagerSlot(self):
        self.switch_selected_view(1)

    def showGreenViewManagerSlot(self):
        self.switch_selected_view(2)

    def showOrangeViewManagerSlot(self):
        self.switch_selected_view(3)

    def showYellowViewManagerSlot(self):
        self.switch_selected_view(4)

    def showCyanViewManagerSlot(self):
        self.switch_selected_view(5)

    def on_fdr_button_clicked(self, t):
        """
        Upon clicking the fdr threshold button the signal with the corresponding t value arives here
        Set the threshold on the current Heatmap volume

        Parameters
        ----------
        t: float
            the t-statistic
        """
        self.modify_layer(Layers.heatmap, 'set_t_threshold', t)

    def volume_changed(self, vol_name):
        """
        When volume is changed from the combobox
        """

        self.modify_layer(Layers.vol1, 'set_volume', vol_name)

    def volume2_changed(self, vol_name):
        """
        When volume is changed from the combobox
        """
        self.modify_layer(Layers.vol2, 'set_volume', vol_name)

    def data_changed(self, vol_name):
        """
        """
        self.modify_layer(Layers.heatmap, 'set_volume', vol_name)
        self.update_connected_components(vol_name)

    def update_connected_components(self, vol_name):
        self.blob_table.clear()
        self.blob_table.setRowCount(0) # clear
        self.blob_table.setHorizontalHeaderLabels(['Count', 'Mean', 'location x:x, y:y, z:z'])

        # set the connected component list
        if vol_name != 'None':
            conn = self.model.getdata(vol_name).connected_components

            for i, (size_mean, bbox) in enumerate(conn.items()):
                self.blob_table.insertRow(i)
                bbox_string = ', '.join(str(x) for x in bbox)
                self.blob_table.setItem(i, 0, QtGui.QTableWidgetItem(str(size_mean[0])))
                self.blob_table.setItem(i, 1, QtGui.QTableWidgetItem(str(size_mean[1])))
                self.blob_table.setItem(i, 2, QtGui.QTableWidgetItem(bbox_string))

        self.blob_table.resizeColumnsToContents()

    def on_connected_table_clicked(self, row, _):
        roi_widget = self.blob_table.item(row, 2)
        roi_str = roi_widget.text()
        roi = [x.strip() for x in roi_str.split(', ')]
        self.roi_signal.emit(roi[0:2], roi[2:4], roi[4:6])

    def vector_changed(self, vol_name):
        self.modify_layer(Layers.vectors, 'set_volume', vol_name)

    def switch_selected_view(self, slice_id):
        """
        update the slice manager with data from a slice view
        :param slice_id: Id of the SliceWidget that this current widget was activated from
        """

        self.ui.pushButtonManagerRed.setStyleSheet("background-color: #FFDEDE")
        self.ui.pushButtonManagerBlue.setStyleSheet("background-color: #D2C9FF")
        self.ui.pushButtonManagerGreen.setStyleSheet("background-color: #C9FFCB")
        self.ui.pushButtonManagerOrange.setStyleSheet("background-color: #FFF3A6")
        self.ui.pushButtonManagerGrey.setStyleSheet("background-color: #FFFFFF")
        self.ui.pushButtonManagerCyan.setStyleSheet("background-color: #ABFFFE")

        if slice_id == 0:
            self.ui.pushButtonManagerRed.setStyleSheet("background-color: red")
        elif slice_id == 1:
            self.ui.pushButtonManagerBlue.setStyleSheet("background-color: blue")
        elif slice_id == 2:
            self.ui.pushButtonManagerGreen.setStyleSheet("background-color: green")
        elif slice_id == 3:
            self.ui.pushButtonManagerOrange.setStyleSheet("background-color: orange")
        elif slice_id == 4:
            self.ui.pushButtonManagerGrey.setStyleSheet("background-color: grey")
        elif slice_id == 5:
            self.ui.pushButtonManagerCyan.setStyleSheet("background-color: cyan")
        self.controller.set_current_view(slice_id)
        self.update()
        self.show()

    def update(self):
        """
        Update the view manager for the current Slice view.
        Called when viewing a new orthogonal view
        """
        self.populate_volume_controls()
        self.populate_heatmap_controls()
        self.populate_vector_controls()


    def refresh_data_comboboxes(self):
        """
        """
        self.populate_volume_controls()
        self.populate_heatmap_controls()
        self.annotations.update()

    def populate_heatmap_controls(self):
        self.ui.comboBoxData.clear()
        self.ui.comboBoxData.addItems(self.model.data_id_list())
        self.ui.comboBoxData.addItem("None")
        self.ui.comboBoxOrientation.setCurrentIndex(self.ui.comboBoxOrientation.findText(
            self.controller.current_orientation().name))
        self.ui.comboBoxLutHeatmap.clear()
        self.ui.comboBoxLutHeatmap.addItems(self.luts.heatmap_lut_list())
        # self.update_connected_components()  TODO: how signal update
        self.update_data_controls()

    def clear_layout(self, layout):
        if layout is not None:
            while layout.count():
                item = layout.takeAt(0)
                widget = item.widget()
                if widget is not None:
                    widget.deleteLater()
                else:
                    self.clear_layout(item.layout())

    def update_fdr_buttons(self):
        """
        When the heatmap volume has been changed, check for the presence of q->t threshold mapping dict and set buttons if present

        """
        slice_layers = self.controller.current_view.layers
        heatmap_vol = slice_layers[Layers.heatmap].vol
        if not heatmap_vol:
            return

        row = 0
        col = 0

        self.clear_layout(self.ui.gridLayoutFdrButtons)

        if heatmap_vol.fdr_thresholds:
            self.ui.labelFdrThresholds.show()
            group = QtGui.QButtonGroup(self)
            for q, t in heatmap_vol.fdr_thresholds.items():
                try:
                    float(t)
                except (ValueError, TypeError):
                    continue
                button = QtGui.QPushButton(str(q))
                group.addButton(button)
                button.clicked.connect(partial(self.on_fdr_button_clicked, t))
                self.ui.gridLayoutFdrButtons.addWidget(button, row, col)
                col += 1
                if col > 5:
                    row += 1
                    col = 0
        else:
            self.ui.labelFdrThresholds.hide()

    def populate_volume_controls(self):
        """
        when a new view manager is activatewd from a new slice view
        :return:
        """
        self.ui.comboBoxVolume.clear()
        self.ui.comboBoxVolume.addItems(self.model.volume_id_list())
        self.ui.comboBoxVolume.addItem("None")
        # do the luts
        self.ui.comboBoxVolumeLut.clear()
        # Lookup table combobox
        self.ui.comboBoxVolumeLut.addItems(self.luts.lut_list())

        self.ui.comboBoxVolume2.clear()
        self.ui.comboBoxVolume2.addItems(self.model.volume_id_list())
        self.ui.comboBoxVolume2.addItem("None")
        # do the luts
        self.ui.comboBoxVolumeLut2.clear()
        # Lookup table combobox
        self.ui.comboBoxVolumeLut2.addItems(self.luts.lut_list())

        self.update_volume_controls()

    def populate_vector_controls(self):
        self.ui.comboBoxVectors.clear()
        self.ui.comboBoxVectors.addItems(self.model.vector_id_list())
        self.ui.comboBoxVectors.addItem("None")
        self.update_vector_controls()

    def update_vector_controls(self):
        vol = self.controller.current_view.layers[Layers.vectors].vol
        if vol:
            self.ui.spinBoxVectorSubsampling.setValue(vol.subsampling)
            self.ui.spinBoxVectorScale.setValue(vol.scale)
            self.ui.comboBoxVectors.setCurrentIndex(self.ui.comboBoxVectors.findText(vol.name))
        else:
             self.ui.comboBoxVectors.setCurrentIndex( self.ui.comboBoxVectors.findText('None'))

    def update_volume_controls(self):
        slice_layers = self.controller.current_view.layers
        vol1 = slice_layers[Layers.vol1].vol

        if vol1:
            min_, max_ = [round(x, 2) for x in vol1.intensity_range()]
            lower, upper = [round(x, 2) for x in vol1.levels]
            self.volume_levels_slider.setMin(min_)
            self.volume_levels_slider.setMax(max_)
            self.volume_levels_slider.setStart(lower)
            self.volume_levels_slider.setEnd(upper)
            self.volume_levels_slider.update()
            self.ui.comboBoxVolume.setCurrentIndex(
                self.ui.comboBoxVolume.findText(vol1.name))
            self.ui.comboBoxVolumeLut.setCurrentIndex(
                self.ui.comboBoxVolumeLut.findText(slice_layers[Layers.vol1].lut[1]))
        else:  # No volume
            self.ui.comboBoxVolume.setCurrentIndex(self.ui.comboBoxVolume.findText('None'))

        vol2 = slice_layers[Layers.vol2].vol
        if vol2:
            min_, max_ = [round(x, 2) for x in vol2.intensity_range()]
            lower, upper = [round(x, 2) for x in vol2.levels]
            self.volume_levels_slider2.setMin(min_)
            self.volume_levels_slider2.setMax(max_)
            self.volume_levels_slider2.setStart(lower)
            self.volume_levels_slider2.setEnd(upper)
            self.volume_levels_slider2.update()
            self.ui.comboBoxVolume2.setCurrentIndex(self.ui.comboBoxVolume2.findText(vol2.name))
            self.ui.comboBoxVolumeLut2.setCurrentIndex(
                self.ui.comboBoxVolumeLut2.findText(slice_layers[Layers.vol2].lut[1]))
        else:  # No second volume overlay
            self.ui.comboBoxVolume2.setCurrentIndex(self.ui.comboBoxVolume2.findText('None'))

    def update_color_scale_bar(self):
        vol = self.controller.current_view.layers[Layers.heatmap].vol
        if vol:
            neg_lower, neg_upper = [round(x, 2) for x in vol.neg_levels]
            pos_lower, pos_upper = [round(x, 2) for x in vol.pos_levels]
            self.colour_bar.update(pos_upper,  pos_lower, neg_upper,neg_lower)

    def update_data_controls(self):
        vol = self.controller.current_view.layers[Layers.heatmap].vol
        self.update_fdr_buttons()
        if vol:
            min_, max_ = [round(x, 2) for x in vol.intensity_range()]
            neg_min_nonzero, pos_min_nonzero = vol.non_zero_mins
            neg_lower, neg_upper = [round(x, 2) for x in vol.neg_levels]
            pos_lower, pos_upper = [round(x, 2) for x in vol.pos_levels]
            # print min_, max_, neg_lower, pos_lower, pos_upper

            # if there are no values for negative or positive stats, we need to inactivate the respective sliders
            # as we can't have sliders with min=0 and max = 0

            self.ui.comboBoxData.setCurrentIndex(self.ui.comboBoxData.findText(vol.name))

            try:
                if pos_upper > 0.0:
                    self.data_levels_positive_slider.setEnabled(True)
                    self.data_levels_positive_slider.setMin(pos_min_nonzero)
                    self.data_levels_positive_slider.setMax(max_)
                    self.data_levels_positive_slider.setStart(pos_lower)
                    self.data_levels_positive_slider.setEnd(pos_upper)
                    self.data_levels_positive_slider.update()
                    pos_bg = 'background: qlineargradient(x1: 0, x2: 1,stop: 0 #222222, stop: 0.5 #FF0000, stop: 1.0 #FFFFFF  );'
                    self.data_levels_positive_slider.handle.setStyleSheet(pos_bg)
                else:  # no positive values, set to grey
                    self.data_levels_positive_slider.setEnabled(False)
                    pos_bg = 'background: qlineargradient(x1: 0.2, x2: 1,stop: 0 #ADADAD, stop: 1  #ADADAD);'
                    self.data_levels_positive_slider.handle.setStyleSheet(pos_bg)
                if neg_lower < 0.0:
                    self.data_levels_negative_slider.setEnabled(True)
                    self.data_levels_negative_slider.setMin(min_)
                    self.data_levels_negative_slider.setMax(neg_min_nonzero)
                    self.data_levels_negative_slider.setStart(neg_lower)
                    self.data_levels_negative_slider.setEnd(neg_upper)
                    self.data_levels_negative_slider.update()
                    neg_bg = 'background: qlineargradient(x1: 0, x2: 1,stop: 0 #FFFFFF, stop: 0.5  #0000FF, stop: 1 #222222);'
                    self.data_levels_negative_slider.handle.setStyleSheet(neg_bg)
                else:  # no negative values, set to grey
                    self.data_levels_negative_slider.setEnabled(False)
                    neg_bg = 'background: qlineargradient(x1: 0.2, x2: 1,stop: 0 #ADADAD, stop: 1  #ADADAD);'
                    self.data_levels_negative_slider.handle.setStyleSheet(neg_bg)
            except TypeError:
                # The slider is raising a type error. Not sure why at the moment
                print('Slider error')
            self.update_color_scale_bar()
            self.ui.doubleSpinBoxNegThresh.setValue(neg_upper)
            self.ui.doubleSpinBoxPosThresh.setValue(pos_lower)
        else:
            self.ui.comboBoxData.setCurrentIndex(self.ui.comboBoxData.findText('None'))

    def modify_layer(self, layer_idx: Layers, method: str, *args):
        if self.link_views:
            for view in self.views.values():
                getattr(view.layers[layer_idx], method)(*args)
        else:
            getattr(self.controller.current_view.layers[layer_idx], method)(*args)

        self.update_slice_views()
        self.update_volume_controls()
        self.update_data_controls()
        self.update_vector_controls()

    def lower_level_volume_changed(self, value):
        self.controller.current_view.layers[Layers.vol1].vol.set_lower_level(value)
        self.update_slice_views()

    def upper_level_volume_changed(self, value):
        self.controller.current_view.layers[Layers.vol1].vol.set_upper_level(value)
        self.update_slice_views()

    def lower_level_volume2_changed(self, value):
        self.controller.current_view.layers[Layers.vol2].vol.set_lower_level(value)
        self.update_slice_views()

    def upper_level_volume2_changed(self, value):
        if self.controller.current_view.layers[Layers.vol2].vol:
            self.controller.current_view.layers[Layers.vol2].vol.set_upper_level(value)
            self.update_slice_views()

    def update_data_lut(self, which, value):
        vol = self.controller.current_view.layers[Layers.heatmap].vol
        if vol:
            if which == 'neg_lower':
                vol.set_lower_negative_lut(value)
            elif which == 'neg_upper':
                vol.set_upper_negative_lut(value)
            elif which == 'pos_lower':
                vol.set_lower_positive_lut(value)
            elif which == 'pos_upper':
                vol.set_upper_positive_lut(value)
        self.update_slice_views()

    def data_negative_lower_changed(self, value):
        self.update_data_lut('neg_lower', value)

    def data_negative_higher_changed(self, value):
        self.update_data_lut('neg_upper', value)
        self.ui.doubleSpinBoxNegThresh.setValue(value)

    def data_positive_lower_changed(self, value):
        self.update_data_lut('pos_lower', value)
        self.ui.doubleSpinBoxPosThresh.setValue(value)

    def data_positive_higher_changed(self, value):
        self.update_data_lut('pos_upper', value)

    # def modify_tstat_lut(self, index, value):
    #     self.update_slice_views()

    def on_vol_lut_changed(self, lut_name):
        self.modify_layer(Layers.vol1, 'set_lut', lut_name)

    def on_vol2_lut_changed(self, lut_name):
        self.modify_layer(Layers.vol2, 'set_lut', lut_name)

    def on_heatmap_lut_changed(self, lut_name):
        """
        The heatmap LUT is stored on the volume not on the layer
        Parameters
        ----------
        lut_name str:
            the name of the LUT
        """
        vol = self.controller.current_view.layers[Layers.heatmap].vol
        vol.set_lut(lut_name)
        self.update_slice_views()

    def update_slice_views(self):
        self.update_color_scale_bar()
        for view in self.views.values():
            view.update_view()

    def clear(self):
        self.blob_table.clear()

    def set_current_sliceview(self, slice_id):
        self.current_slice_view = self.model.slice_views[slice_id]

    def on_link_views(self, checked):
        self.link_views = checked

    @property
    def link_views(self):
        return self._link_views

    @link_views.setter
    def link_views(self, checked):
        self._link_views = checked
        self.ui.checkBoxLinkViews.setChecked(checked)

    def left_view_visibility(self, checked):
        if checked:
            self.views[0].show()
            if self.six_views_visible:
                self.views[3].show()
        else:
            self.views[0].hide()
            if self.six_views_visible:
                self.views[3].hide()

    def central_view_visibility(self, checked):
        if checked:
            self.views[1].show()
            if self.six_views_visible:
                self.views[4].show()
        else:
            self.views[1].hide()
            if self.six_views_visible:
                self.views[4].hide()

    def right_view_visibility(self, checked):
        if checked:
            self.views[2].show()
            if self.six_views_visible:
                self.views[5].show()
        else:
            self.views[2].hide()
            if self.six_views_visible:
                self.views[5].hide()

    def show2Rows(self, checked):
        if checked:
            self.six_views_visible = True
            self.views[3].show()
            self.views[4].show()
            self.views[5].show()
            self.ui.pushButtonManagerOrange.show()
            self.ui.pushButtonManagerGrey.show()
            self.ui.pushButtonManagerCyan.show()
        else:
            self.six_views_visible = False
            self.views[3].hide()
            self.views[4].hide()
            self.views[5].hide()
            self.ui.pushButtonManagerOrange.hide()
            self.ui.pushButtonManagerGrey.hide()
            self.ui.pushButtonManagerCyan.hide()
        self.controller.update_slice_views()

    def on_interpolate(self, checked):
        self.model.set_interpolation(checked)
        self.update_slice_views()

    def on_orientation(self, orientation: str):
        """
        Activated when the orientation combobox is changed
        :return:
        """
        # convert the str to an enum member

        orientation = Orientation[orientation]

        if self.link_views:
            for v in self.views.values():
                v.set_orientation(orientation)
        else:
            v = self.controller.current_view
            v.set_orientation(orientation)


class ColorScaleBar(object):
    def __init__(self, layout, neg_lut, pos_lut):
        self.color_scale_widget = pg.GraphicsLayoutWidget()
        self.color_scale_widget.setMaximumSize(150, 300)
        self.color_scale_widget.setMinimumSize(150, 300)
        self.color_scale_widget.hide()
        self.layout = layout
        self.layout.addWidget(self.color_scale_widget)
        self.color_scale_view = self.color_scale_widget.addViewBox(row=0, col=0, enableMouse=False, lockAspect=True)
        self.font = QtGui.QFont('Arial', 13, QtGui.QFont.Bold)
        self.neg_lut = neg_lut
        self.pos_lut = pos_lut
        self.pushButtonInvertColor = QtGui.QPushButton('Invert color')
        self.pushButtonInvertColor.clicked.connect(self.on_invert)
        self.layout.addWidget(self.pushButtonInvertColor)
        self.pushButtonInvertColor.hide()
        self.neg_text_coords = [10, 13]
        self.pos_text_coords = [10, 210]
        self.tstat_label_coords = [-5, 140]

    def update(self, max_pos, min_neg, min_pos, max_neg):
 
        if self.color_scale_view:
            self.color_scale_widget.removeItem(self.color_scale_view)
        self.color_scale_view = self.color_scale_widget.addViewBox(row=0, col=0, enableMouse=False, lockAspect=True)

        self.min_pos_text = pg.TextItem(str(round(min_pos, 2)))
        self.min_neg_text = pg.TextItem(str(round(min_neg, 2)))

        self.max_pos_text = pg.TextItem(str(round(max_pos, 2)))
        self.max_neg_text = pg.TextItem(str(round(max_neg, 2)))

        rgba_pos = []
        for x in range(0, self.pos_lut.shape[1]):
            col = self.pos_lut[:, x]
            slice_ = np.array([col])
            rgba_pos.append(slice_.T)

        rgba_neg = []
        for x in range(0, self.neg_lut.shape[1]):
            col = self.neg_lut[:, x]
            slice_ = np.array([col])
            rgba_neg.append(slice_.T)

        pos_img = np.dstack(tuple(rgba_pos)).astype(np.uint8)
        neg_img = np.dstack(tuple(rgba_neg)).astype(np.uint8)

        full_img = np.rot90(np.vstack((neg_img, pos_img)))

        remove_padding = 0
        # remove the black section of the colormap from the negative scale
        remove_start = int(full_img.shape[1] / 2 - remove_padding)
        remove_end = int(full_img.shape[1] / 2 + remove_padding)
        remove_range = range(remove_start, remove_end)

        full_img = np.delete(full_img, remove_range, axis=1)

        ii = pg.ImageItem(full_img)
        ii.setRect(QtCore.QRect(0, 0, 10, 200))

        self.color_scale_view.addItem(self.max_pos_text)
        self.color_scale_view.addItem(self.min_neg_text)
        self.color_scale_view.addItem(self.min_pos_text)
        self.color_scale_view.addItem(self.max_neg_text)
        self.max_pos_text.setFont(self.font)
        self.min_neg_text.setFont(self.font)
        self.min_pos_text.setFont(self.font)
        self.max_neg_text.setFont(self.font)

        self.max_neg_text.setPos(*self.neg_text_coords)
        self.max_pos_text.setPos(*self.pos_text_coords)
        #min_neg_text.setPos(10, 60)

        self.color_scale_view.addItem(ii)

        # Find how far up the color map the minimum positive  is
        min_pos_y = (full_img.shape[1] / 2) + 20
        min_pos_y_mapped = self.color_scale_view.mapFromItemToView(ii, QtCore.QPointF(40, min_pos_y))
        self.min_pos_text.setPos(9, min_pos_y_mapped.y())

        min_neg_y = full_img.shape[1] / 2 + 80
        min_neg_y_mapped = self.color_scale_view.mapFromItemToView(ii, QtCore.QPointF(40, min_neg_y))
        self.min_neg_text.setPos(13, min_neg_y_mapped.y())

        self.label = pg.TextItem('t-statistics', angle=-90)
        self.label.setFont(self.font)
        self.color = 'white'
        #self.label.setText('t-statistics', self.color)
        self.color_scale_view.addItem(self.label)
        self.label.setPos(*self.tstat_label_coords)
        self.inverted = False


    def redraw(self):
        pass

    def on_invert(self):
        if self.inverted:
            self.inverted = False
            self.label.textItem.setDefaultTextColor(QtGui.QColor(0, 0, 0))
            self.max_pos_text.textItem.setDefaultTextColor(QtGui.QColor(0, 0, 0))
            self.min_pos_text.textItem.setDefaultTextColor(QtGui.QColor(0, 0, 0))
            self.max_neg_text.textItem.setDefaultTextColor(QtGui.QColor(0, 0, 0))
            self.min_neg_text.textItem.setDefaultTextColor(QtGui.QColor(0, 0, 0))
            self.color_scale_widget.setBackground(QtGui.QColor(255, 255, 255))

        else:
            self.inverted = True
            self.label.textItem.setDefaultTextColor(QtGui.QColor(255, 255, 255))
            self.max_pos_text.textItem.setDefaultTextColor(QtGui.QColor(255, 255, 255))
            self.min_pos_text.textItem.setDefaultTextColor(QtGui.QColor(255, 255, 255))
            self.max_neg_text.textItem.setDefaultTextColor(QtGui.QColor(255, 255, 255))
            self.min_neg_text.textItem.setDefaultTextColor(QtGui.QColor(255, 255, 255))
            self.color_scale_widget.setBackground(QtGui.QColor(0, 0, 0))

    def hide(self):
        self.pushButtonInvertColor.hide()
        self.color_scale_widget.hide()
        self.pushButtonInvertColor.hide()

    def show(self):
        self.pushButtonInvertColor.show()
        self.color_scale_widget.show()
        self.pushButtonInvertColor.show()

    def set_minimum_value(self, value):
        pass

    def set_maximum_value(self, value):
        pass